@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: cons.c,v 1.29 1999/07/08 18:02:48 pvmsrc Exp $";
+	"$Id: cons.c,v 1.31 2001/02/07 23:10:09 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -37,6 +37,15 @@ static char rcsid[] =
  *	machine and other tasks.
  *
  * $Log: cons.c,v $
+ * Revision 1.31  2001/02/07 23:10:09  pvmsrc
+ * CYGWIN Changes.
+ * (Spanker=kohl)
+ *
+ * Revision 1.30  2000/02/15 18:05:54  pvmsrc
+ * Modified console to intialize trace mask at startup.
+ * 	- use "trace xpvm" command via trace_cmd() interface.
+ * (Spanker=kohl)
+ *
  * Revision 1.29  1999/07/08 18:02:48  pvmsrc
  * Fixed up crummy readline garbage.
  * 	- added omitted "char *line" argument to rl_got_line(),
@@ -218,8 +227,11 @@ static char rcsid[] =
 
 #ifdef HASREADLINE
 
-#include<readline/readline.h>
-#include<readline/history.h>
+#include <readline/readline.h>
+
+#ifndef CYGWIN
+#include <readline/history.h>
+#endif
 
 char **command_completion();
 
@@ -293,6 +305,8 @@ main(argc, argv)
 	struct pvmminfo minfo;
 	struct timeval *timeout;
 	int more;
+	int ac;
+	char *av[128];
 
 #ifdef WIN32
 
@@ -309,6 +323,39 @@ main(argc, argv)
 	}
 */
 
+#endif
+
+#ifdef CYGWIN
+
+	/*
+		To run properly, the PVM console has to be run
+		in CYGWIN's tty mode
+	*/
+
+	{
+		char *cygwin_env;
+		int cygwin_tty = 0;
+	
+		cygwin_env = getenv("CYGWIN");
+		if (cygwin_env==NULL) 
+			cygwin_env = getenv("CYGWIN32");
+		if (cygwin_env!=NULL) {
+			cygwin_tty  = (strstr(cygwin_env, "tty") == NULL);
+			cygwin_tty |= (strstr(cygwin_env, "TTY") == NULL);
+		}
+		if (!cygwin_tty) {
+			fprintf(stderr, 
+		 		"PVM will not run properly if not in tty mode.\n");
+			fprintf(stderr, 
+		 		"Please set the following before calling PVM again:\n");
+			fprintf(stderr, 
+		 		"SET CYGWIN=tty   (Cygwin B20)\n");
+			fprintf(stderr, 
+		 		"SET CYGWIN32=tty (Cygwin B19)\n");
+			exit(1);
+		}
+	}		
+
 #endif
 
 	pvm_setopt(PvmResvTids, 1);
@@ -349,6 +396,12 @@ main(argc, argv)
 #endif
 	job_init();
 
+	/* set default trace mask to xpvm */
+	sprintf(cmd, "trace xpvm");
+	ac = sizeof(av)/sizeof(av[0]) - 1;
+	if (!acav(cmd, &ac, av) && ac)
+		trace_cmd(ac, av);
+
 	pvm_notify(PvmHostAdd, HostsAddedTag, -1, (int*)0);
 
 	BZERO(&minfo, sizeof(minfo));
