@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: job.c,v 1.8 1999/07/08 18:59:41 kohl Exp $";
+	"$Id: job.c,v 1.10 2000/02/15 17:16:47 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -36,6 +36,16 @@ static char rcsid[] =
  *	Job descriptor / output gathering code.
  *
  * $Log: job.c,v $
+ * Revision 1.10  2000/02/15 17:16:47  pvmsrc
+ * *Really* removed old checktrace() routine.
+ * 	- hasn't been called for years...  duh...
+ * (Spanker=kohl)
+ *
+ * Revision 1.9  2000/02/15 17:06:20  pvmsrc
+ * Modified dumping of task output / trace events.
+ * 	- key off of PvmShowTids option, leave off task id prefix if set.
+ * (Spanker=kohl)
+ *
  * Revision 1.8  1999/07/08 18:59:41  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -244,6 +254,9 @@ checkoutput(jp, cc, len, cod, src)
 	char *p;
 	int ptid;
 	struct obuf *op;
+	int show;
+
+	show = pvm_getopt( PvmShowTids );
 
 	pvm_upkint(&tid, 1, 1);
 	pvm_upkint(&n, 1, 1);
@@ -273,9 +286,11 @@ checkoutput(jp, cc, len, cod, src)
 		op->o_len += n;
 		while (p = CINDEX(p, '\n')) {
 			*p++ = 0;
+			if ( show )
+				fprintf((jp->j_ff ? jp->j_ff : stdout),
+						"[%d:t%x] ", jp->j_jid - joboffset, tid);
 			fprintf((jp->j_ff ? jp->j_ff : stdout),
-					"[%d:t%x] %s\n", jp->j_jid - joboffset,
-					tid, op->o_buf);
+					"%s\n", op->o_buf);
 			op->o_len -= p - op->o_buf;
 			BCOPY(p, op->o_buf, op->o_len);
 			p = op->o_buf;
@@ -291,15 +306,19 @@ checkoutput(jp, cc, len, cod, src)
 				goto hork;
 			}
 			if (op->o_len > 0) {
+				if ( show )
+					fprintf((jp->j_ff ? jp->j_ff : stdout),
+							"[%d:t%x] ", jp->j_jid - joboffset, tid);
 				fprintf((jp->j_ff ? jp->j_ff : stdout),
-						"[%d:t%x] %s\n", jp->j_jid - joboffset,
-						tid, op->o_buf);
+						"%s\n", op->o_buf);
 				op->o_len = 0;
 			}
 			op->o_flag |= GOTEOF;
 			if (op->o_flag == (GOTSPAWN|GOTEOF)) {
-				fprintf((jp->j_ff ? jp->j_ff : stdout),
-						"[%d:t%x] EOF\n", jp->j_jid - joboffset, tid);
+				if ( show )
+					fprintf((jp->j_ff ? jp->j_ff : stdout),
+							"[%d:t%x] EOF\n", jp->j_jid - joboffset,
+							tid);
 				obuf_free(jp, op);
 			}
 			break;
@@ -312,8 +331,10 @@ checkoutput(jp, cc, len, cod, src)
 */
 			op->o_flag |= GOTSPAWN;
 			if (op->o_flag == (GOTSPAWN|GOTEOF)) {
-				fprintf((jp->j_ff ? jp->j_ff : stdout),
-						"[%d:t%x] EOF\n", jp->j_jid - joboffset, tid);
+				if ( show )
+					fprintf((jp->j_ff ? jp->j_ff : stdout),
+							"[%d:t%x] EOF\n", jp->j_jid - joboffset,
+							tid);
 				obuf_free(jp, op);
 			}
 
@@ -340,88 +361,3 @@ hork:
 }
 
 
-checktrace(jp, cc, len, cod, src)
-	struct job *jp;
-	int cc;					/* message mid */
-	int len;
-	int cod;
-	int src;
-{
-	int tid;					/* task */
-	int kind;					/* length or event code */
-	int sec, usec;
-	char *p;
-	int ptid;
-	struct obuf *op;
-
-	pvm_upkint(&sec, 1, 1);
-	pvm_upkint(&usec, 1, 1);
-	pvm_upkint(&tid, 1, 1);
-	pvm_upkint(&kind, 1, 1);
-
-	if (kind >= 0 && kind <= TEV_MAX) {
-
-		if (!(op = obuf_find(jp, tid)))
-			op = obuf_new(jp, tid);
-
-		op->o_flag |= GOTSPAWN;
-
-		if (!(op = obuf_find(jp, tid))) {
-			printf("Bogus message, job %d has no task t%x\n",
-					jp->j_jid - joboffset, tid);
-			goto hork;
-		}
-
-		fprintf((jp->j_ff ? jp->j_ff : stdout), "[T%d:t%x] %d.%06d ",
-				jp->j_jid - joboffset, tid, sec, usec);
-
-		fprintf((jp->j_ff ? jp->j_ff : stdout), "\n");
-
-		switch (kind) {
-
-		case TEV_ENDTASK:	/* EOF */
-			if (!(op = obuf_find(jp, tid))) {
-				printf("Bogus message, job %d has no task t%x\n",
-						jp->j_jid - joboffset, tid);
-				goto hork;
-			}
-			op->o_flag |= GOTEOF;
-			if (op->o_flag == (GOTSPAWN|GOTEOF)) {
-				fprintf((jp->j_ff ? jp->j_ff : stdout),
-						"[T%d:t%x] END\n", jp->j_jid - joboffset, tid);
-				obuf_free(jp, op);
-			}
-			break;
-
-		case TEV_SPNTASK:	/* tm_spawn */
-			if (!(op = obuf_find(jp, tid)))
-				op = obuf_new(jp, tid);
-			op->o_flag |= GOTSPAWN;
-			if (op->o_flag == (GOTSPAWN|GOTEOF)) {
-				fprintf((jp->j_ff ? jp->j_ff : stdout),
-						"[T%d:t%x] END\n", jp->j_jid - joboffset, tid);
-				obuf_free(jp, op);
-			}
-
-			break;
-
-		case TEV_NEWTASK:	/* dm_exec */
-			if (!(op = obuf_find(jp, tid)))
-				op = obuf_new(jp, tid);
-			break;
-
-		default:
-			break;
-		}
-
-	} else {
-		printf("Bogus message from job %d task t%x\n",
-				jp->j_jid - joboffset, tid);
-	}
-
-hork:
-	pvm_freebuf(cc);
-	return 0;
-}
-
-
