@@ -1,4 +1,6 @@
-
+c
+c $Id: master1.f,v 1.2 1997/06/26 19:42:55 pvmsrc Exp $
+c
       program master1 
       include '../include/fpvm3.h'
 c ---------------------------------------------------------
@@ -17,28 +19,22 @@ c     Enroll this program in PVM
       call pvmfmytid( mytid )
 
 c     Set number of slaves to spawn. 
-c     Can't do standard input if master started with spawn so
-c     just set nproc = number of hosts in this case. Else ask for nproc.
-
-      call pvmfparent( iptid )
-      if( iptid .gt. 1 ) then
-        call pvmfconfig( nhost, narch, dtid, host, arch, speed, info )
-        nproc = nhost
-        if( nproc .gt. 32 ) nproc = 32
-      else
-        print *,'How many slave programs (1-32)?'
-        read *, nproc
-      endif
+c     Note standard input cannot be read if spawned from console, so just
+c     derive nprocs from the VM config
+      call pvmfconfig( nhost, narch, dtid, host, arch, speed, info )
+      nproc = nhost * 3
+      if( nproc .gt. 32 ) nproc = 32
 c
 c     Initiate nproc instances of slave1 program 
 c     If arch is set to '*' then ANY configured machine is acceptable
-      nodename = 'slave1'
+      write(6,6000) nproc
+6000  format(' ','Spawning ', I4, ' tasks ...')
+6001  format(' ',25X, 'SUCCESSFUL')
+      nodename = 'fslave1'
       arch = '*'
       call pvmfspawn( nodename, PVMDEFAULT, arch, nproc, tids, numt )
 
-c     Print out task IDs of spawned tasks and check for problems
-      do 100 i=0, nproc-1
-         print *,'tid',i,tids(i)
+c     Check for problems
 100   continue
       if( numt .lt. nproc ) then
          print *, 'trouble spawning ',nodename
@@ -46,9 +42,11 @@ c     Print out task IDs of spawned tasks and check for problems
          call shutdown( numt, tids )
       endif
 
+      write(6,6001)
+
 c ------- Begin user program -------- 
 
-      n = 10
+      n = 100
 c     Initiate data array
       do 20 i=1,n
          data(i) = 1
@@ -69,7 +67,14 @@ c     wait for results from nodes
          call pvmfrecv( -1, msgtype, info ) 
          call pvmfunpack( INTEGER4, who, 1, 1, info )
          call pvmfunpack( REAL8, result(who+1), 1, 1, info )
-         print *, 'I got',result(who+1), '  from', who 
+	     if (who .eq. 0) then
+            write(6,6002)  result(who+1), who, (nproc - 1) * 100.0
+         else
+			write(6,6002) result(who+1), who,  (2 * who - 1) * 100.0
+         endif
+ 6002    format(' ', 'I got ', F15.7, ' from', I4,
+     +                ' (expected ',F15.7,' )')
+	
  30   continue 
 
 c --------- End user program -------- 
@@ -88,5 +93,6 @@ c
          call pvmfkill( tids(i), info )
   10  continue
       call pvmfexit( info )
+	  stop
       return
       end
