@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: mbox.c,v 1.9 1999/03/15 19:04:19 pvmsrc Exp $";
+	"$Id: mbox.c,v 1.11 2001/06/27 23:20:51 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -212,6 +212,8 @@ char *argv[];
 
 	char *arg;
 
+	int NINDICES = 10;
+
 	int do_noreset;
 	int do_favail;
 	int do_unpack;
@@ -291,6 +293,9 @@ char *argv[];
 			do_unpack = 0;
 		}
 
+		else if ( !strncmp( arg, "nindices=", 9 ) )
+			NINDICES = atoi( arg + 9 );
+
 		else if ( !strcmp( arg, "help" ) )
 			usage();
 
@@ -370,7 +375,7 @@ char *argv[];
 
 			if ( ncmds == 1 )
 			{
-				for ( i=0 ; i < 10 ; i++ )
+				for ( i=0 ; i < NINDICES ; i++ )
 				{
 					cc = check_msg( CLASS_NAME, i, flags,
 						bufout, &tmp, do_unpack );
@@ -415,7 +420,7 @@ char *argv[];
 
 			if ( ncmds == 1 )
 			{
-				for ( i=0 ; i < 10 ; i++ )
+				for ( i=0 ; i < NINDICES ; i++ )
 				{
 					cc = pvm_delinfo( CLASS_NAME, i, flags );
 
@@ -853,6 +858,73 @@ char *argv[];
 
 			ret++;
 		}
+
+		else if ( !strcmp( cmds[0], "direct" ) )
+		{
+			set_flags( &flags, 0 /*do_pers*/, 1 /*do_favail*/, do_owrit,
+				1 /*do_minst*/, do_rddel, do_wait );
+
+			for ( i=0 ; i < 3 ; i++ )
+			{
+				sprintf( bufin,
+					"This Message Left Here By Mbox (%d)", i );
+
+				index = stuff_msg( CLASS_NAME, flags, bufin, mytid );
+
+				if ( index < 0 )
+					exit( -1 );
+
+				cc = check_msg( CLASS_NAME, index, 0, bufout, &tmp,
+					do_unpack );
+
+				if ( cc < 0 )
+					exit( -1 );
+		
+				if ( strcmp( bufout, bufin ) )
+				{
+					printf(
+						"String Mismatch Error:  \"%s\"  !=  \"%s\"\n",
+						bufout, bufin );
+				}
+
+				else if ( tmp != mytid )
+				{
+					printf( "Number Mismatch Error:  %d  !=  %d\n",
+						tmp, mytid );
+				}
+
+				else
+				{
+					printf( "Retrieved Message is a Match.\n" );
+
+					ret++;
+				}
+			}
+
+			if ( do_sleep )
+				pvmsleep( do_sleep );
+
+			set_flags( &flags, 0 /*do_pers*/, 0 /*do_favail*/, do_owrit,
+				0 /*do_minst*/, do_rddel, do_wait );
+
+			sprintf( bufin,
+				"This Message Left Here By Mbox (X)" );
+
+			index = stuff_msg( CLASS_NAME,
+				flags | PvmMboxDirectIndex( 1 ), bufin, mytid );
+
+			if ( index < 0 )
+				exit( -1 );
+
+			for ( i=0 ; i < 3 ; i++ )
+			{
+				cc = check_msg( CLASS_NAME, i, 0, bufout, &tmp,
+					do_unpack );
+
+				if ( cc < 0 )
+					exit( -1 );
+			}
+		}
 	}
 
 	if ( do_exit )
@@ -1007,8 +1079,8 @@ usage()
 	printf( "usage:  mbox [pers] [favail] [owrit] [minst] [rddel]\n" );
 	printf( "\t[sleep[=<secs>]] [name=<class_name>]\n" );
 	printf( "\t[query [<index>]] [wait] [noreset] [noexit]\n" );
-	printf( "\t[info [<pattern>]] [delete [<index>]]\n" );
-	printf( "\t[default] [default33] [help]\n" );
+	printf( "\t[info [<pattern>]] [delete [<index>]] [direct]\n" );
+	printf( "\t[default] [default33] [nindices=<n>] [help]\n" );
 	printf( "\n" );
 
 	exit( 0 );
