@@ -1,4 +1,35 @@
 
+static char rcsid[] =
+	"$Id: timing.c,v 1.4 1998/10/01 21:20:55 pvmsrc Exp $";
+
+/*
+ *         PVM version 3.4:  Parallel Virtual Machine System
+ *               University of Tennessee, Knoxville TN.
+ *           Oak Ridge National Laboratory, Oak Ridge TN.
+ *                   Emory University, Atlanta GA.
+ *      Authors:  J. J. Dongarra, G. E. Fagg, M. Fischer
+ *          G. A. Geist, J. A. Kohl, R. J. Manchek, P. Mucci,
+ *         P. M. Papadopoulos, S. L. Scott, and V. S. Sunderam
+ *                   (C) 1997 All Rights Reserved
+ *
+ *                              NOTICE
+ *
+ * Permission to use, copy, modify, and distribute this software and
+ * its documentation for any purpose and without fee is hereby granted
+ * provided that the above copyright notice appear in all copies and
+ * that both the copyright notice and this permission notice appear in
+ * supporting documentation.
+ *
+ * Neither the Institutions (Emory University, Oak Ridge National
+ * Laboratory, and University of Tennessee) nor the Authors make any
+ * representations about the suitability of this software for any
+ * purpose.  This software is provided ``as is'' without express or
+ * implied warranty.
+ *
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
+ */
+
 /*
 *	timing.c
 *
@@ -14,13 +45,18 @@
 *   6 Mar 1994  Geist  - synch tasks and add direct route
 */
 
+#ifdef HASSTDLIB
+#include <stdlib.h>
+#endif
 #include <stdio.h>
-#include <sys/time.h>
-#include <time.h>
 
+#ifndef WIN32
+#include <sys/time.h>
+#endif
+
+#include <time.h>
 #include <sys/types.h>
 #include <fcntl.h>
-#include <stdio.h>
 #include <math.h>
 #include "pvm3.h"
 
@@ -84,7 +120,7 @@ main(argc, argv)
 		}
 
 		if (pvm_recv(-1, -1) < 0) {
-			fprintf(stderr, "recv error%d\n" );
+			pvm_perror( "recv error" );
 			goto bail;
 		}
 
@@ -125,7 +161,7 @@ main(argc, argv)
 			}
 
 			if (pvm_recv(-1, -1) < 0) {
-				fprintf(stderr, "recv error%d\n" );
+				pvm_perror( "recv error" );
 				goto bail;
 			}
 
@@ -150,6 +186,11 @@ main(argc, argv)
 			(numint * 4) / (double)at2);
 	}
 
+	/* we have to do this because the last message might be taking
+	*  up all the shared memory pages.
+	*/
+	pvm_freebuf(pvm_getsbuf());
+
 	puts("\ndone");
 
 bail:
