@@ -1,5 +1,5 @@
 
-/* $Id: pvm3.h,v 1.40 1999/08/19 16:07:14 pvmsrc Exp $ */
+/* $Id: pvm3.h,v 1.52 2004/09/08 19:52:43 pvmsrc Exp $ */
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -35,6 +35,71 @@
  *	Libpvm3 includes.
  *
  * $Log: pvm3.h,v $
+ * Revision 1.52  2004/09/08 19:52:43  pvmsrc
+ * Upped to PVM version 3.4.5 (how sequential... :)
+ * (Spanker=kohl)
+ *
+ * Revision 1.51  2001/09/27 21:24:18  pvmsrc
+ * Increased default PVMTMPNAMLEN to 64...
+ * 	- suggested by BEOSCYLD / Joe Vitale <vitale@scyld.com>
+ * 	- seems like a good idea.  :-)
+ * (Spanker=kohl)
+ *
+ * Revision 1.50  2001/09/27 18:24:39  pvmsrc
+ * Upped version to 3.4.4.
+ * (Spanker=kohl)
+ *
+ * Revision 1.49  2001/09/26 21:55:51  pvmsrc
+ * Oops...  fixed function proto for pvmtmpnam()...
+ * 	- leftover bogus cruft...
+ * (Spanker=kohl)
+ *
+ * Revision 1.48  2001/09/25 21:17:49  pvmsrc
+ * Minor TMPNAMFUN()/tmpnam() cleanup.
+ * 	- moved macro def to pvm3.h, renamed PVMTNPMAN().
+ * 	- same for LEN_OF_TMP_NAM -> PVMTMPNAMLEN.
+ * 	- mostly a huge waste of time, since *both* tmpnam() & mktemp()
+ * 		produce the same "dangerous" warning message in Linux/gcc...
+ * 	- damn.
+ * (Spanker=kohl)
+ *
+ * Revision 1.47  2001/06/18 18:38:23  pvmsrc
+ * Added missing function prototype for pvm_freezegroup()...
+ * (Spanker=kohl)
+ *
+ * Revision 1.46  2000/02/16 22:50:01  pvmsrc
+ * Upped version to 3.4.3 so I don't forget later.
+ * (Spanker=kohl)
+ *
+ * Revision 1.45  2000/02/10 23:53:09  pvmsrc
+ * Added new PvmIPLoopback error code.
+ * 	- Master Host IP Address tied to Loopback.
+ * (Spanker=kohl)
+ *
+ * Revision 1.44  1999/11/08 17:21:32  pvmsrc
+ * Added PvmMboxDirectIndex stuff:
+ * 	- PvmMboxMaxFlag: capped regular flags to squeeze in direct index...
+ * 	- PvmMboxDirectIndexShift: # of bits to shift index in flags
+ * 	- PvmMboxMaxDirectIndex: max direct index value, for error checking
+ * 	- PvmMboxDirectIndex(): macro to convert direct index into flag bits
+ * 	- PvmMboxDirectIndexOf(): macro to decode direct index from flags
+ * (Spanker=kohl)
+ *
+ * Revision 1.43  1999/10/27 18:48:40  pvmsrc
+ * Fixed (yet again) the prototype for pvm_recvf().
+ * 	- hopefully got it right this time...  :-Q
+ * (Spanker=kohl)
+ *
+ * Revision 1.42  1999/10/22 13:34:55  pvmsrc
+ * Fixed prototype for pvm_recvf().
+ * 	- reported by "Ed D'Azevedo" <efdazedo@alcor.epm.ornl.gov>.
+ * (Spanker=kohl)
+ *
+ * Revision 1.41  1999/10/18 21:22:00  pvmsrc
+ * Added function prototypes for:
+ * 	- pvm_pkmesg(), pvm_pkmesgbody(), pvm_upkmesg().
+ * (Spanker=kohl)
+ *
  * Revision 1.40  1999/08/19 16:07:14  pvmsrc
  * Upped version to 3.4.2.
  * (Spanker=kohl)
@@ -211,10 +276,10 @@
 *	Release version
 */
 
-#define	PVM_VER				"3.4.2"
+#define	PVM_VER				"3.4.5"
 #define	PVM_MAJOR_VERSION	3
 #define	PVM_MINOR_VERSION	4
-#define PVM_PATCH_VERSION	2
+#define PVM_PATCH_VERSION	5
 
 #ifndef WIN32
 #include	<sys/time.h>
@@ -328,7 +393,27 @@
 #define	PvmMboxReadAndDelete	16	/* atomic read / delete */
 									/* requires read & delete rights */
 #define	PvmMboxWaitForInfo		32	/* for blocking recvinfo */
+#define PvmMboxMaxFlag			512	/* maximum mbox flag bit value */
 
+#define PvmMboxDirectIndexShift	10	/* log2(PvmMboxMaxFlag) + 1 */
+
+#define PvmMboxMaxDirectIndex \
+	( (unsigned) 1 << ( 31 - PvmMboxDirectIndexShift ) )
+
+#define PvmMboxDirectIndexOf( _flags ) \
+	( ( (_flags) & \
+		( ( PvmMboxMaxDirectIndex - 1 ) << PvmMboxDirectIndexShift ) ) \
+		>> PvmMboxDirectIndexShift )
+
+#define PvmMboxDirectIndex( _index ) \
+	( ( (_index) >= PvmMboxMaxDirectIndex ) ? \
+		( fprintf( stderr, "Mbox Direct Index Too Large: %d >= %d\n", \
+			(_index), PvmMboxMaxDirectIndex ), 0 ) \
+		: ( (_index) << PvmMboxDirectIndexShift ) )
+
+/*
+*	pre-defined system message mailbox classes
+*/
 
 #define PVMNORESETCLASS		"###_PVM_NO_RESET_###"
 
@@ -376,6 +461,7 @@
 #define	PvmExists		-33	/* Already exists */
 #define	PvmHostrNMstr	-34	/* Hoster run on non-master host */
 #define	PvmParentNotSet	-35	/* Spawning parent set PvmNoSpawnParent */
+#define	PvmIPLoopback	-36	/* Master Host's IP is Loopback */
 
 /*
 *	crusty error constants from 3.3, now redefined...
@@ -452,6 +538,29 @@ struct pvmmboxinfo {
 	int *mi_flags;			/* mbox entry flags */
 };
 
+/*
+*	temporary file name function / macro
+*	(since there are so many choices, but so little time,
+*	and since gcc now complains about tmpnam()... :-Q)
+*/
+
+#ifdef  NOTMPNAM
+#define PVMTMPNAMFUN(x)		pvmtmpnam(x)
+#define PVMTMPNAMLEN		64
+#else
+#define PVMTMPNAMFUN(x)		tmpnam(x)
+#ifdef L_tmpnam
+#define PVMTMPNAMLEN		L_tmpnam
+#else
+#define PVMTMPNAMLEN		64
+#endif
+#endif
+
+
+/*
+*	function prototypes
+*/
+
 #ifdef __ProtoGlarp__
 #undef __ProtoGlarp__
 #endif
@@ -465,6 +574,12 @@ struct pvmmboxinfo {
 extern "C" {
 #endif
 
+#ifdef  NOTMPNAM
+char *pvmtmpnam		__ProtoGlarp__(( char * ));
+#endif
+char *pvmgetrsh		__ProtoGlarp__(( void ));
+char *pvmgettmp		__ProtoGlarp__(( void ));
+
 int pvm_addhosts	__ProtoGlarp__(( char **, int, int * ));
 int pvm_addmhf		__ProtoGlarp__(( int, int, int, int (*) (int) ));
 int pvm_archcode	__ProtoGlarp__(( char * ));
@@ -483,6 +598,7 @@ int pvm_exit		__ProtoGlarp__(( void ));
 int pvm_export		__ProtoGlarp__(( char * ));
 int pvm_freebuf		__ProtoGlarp__(( int ));
 int pvm_freecontext	__ProtoGlarp__(( int ));
+int pvm_freezegroup	__ProtoGlarp__(( char *, int ));
 int pvm_gather		__ProtoGlarp__(( void*, void*,
 										int, int, int, char*, int));
 int pvm_getcontext	__ProtoGlarp__(( void ));
@@ -527,6 +643,8 @@ int pvm_pkstr		__ProtoGlarp__(( char * ));
 int pvm_pkuint		__ProtoGlarp__(( unsigned int *, int, int ));
 int pvm_pkulong		__ProtoGlarp__(( unsigned long *, int, int ));
 int pvm_pkushort	__ProtoGlarp__(( unsigned short *, int, int ));
+int pvm_pkmesg		__ProtoGlarp__(( int ));
+int pvm_pkmesgbody	__ProtoGlarp__(( int ));
 int pvm_precv		__ProtoGlarp__(( int, int,
 									void *, int, int,
 									int *, int *, int * ));
@@ -536,7 +654,8 @@ int pvm_psend		__ProtoGlarp__(( int, int,
 int pvm_pstat		__ProtoGlarp__(( int ));
 int pvm_putinfo		__ProtoGlarp__(( char *, int, int ));
 int pvm_recv		__ProtoGlarp__(( int, int ));
-int (*pvm_recvf		__ProtoGlarp__(( int (*)(int, int, int) )) )();
+int (*pvm_recvf		__ProtoGlarp__(( int (*)(int, int, int) )) )
+	__ProtoGlarp__(( int (*)(int, int, int) ));
 int pvm_recvinfo	__ProtoGlarp__(( char *, int, int ));
 int pvm_reduce		__ProtoGlarp__(( void (*)(int*, void*, void*, int*, int*),
 									void *, int,
@@ -593,6 +712,7 @@ int pvm_upkstr		__ProtoGlarp__(( char * ));
 int pvm_upkuint		__ProtoGlarp__(( unsigned int *, int, int ));
 int pvm_upkulong	__ProtoGlarp__(( unsigned long *, int, int ));
 int pvm_upkushort	__ProtoGlarp__(( unsigned short *, int, int ));
+int pvm_upkmesg		__ProtoGlarp__(( void ));
 char *pvm_version	__ProtoGlarp__(( void ));
 
 /*
@@ -605,7 +725,7 @@ int pvm_getmwid		__ProtoGlarp__(( int ));
 int pvm_setmwid		__ProtoGlarp__(( int, int ));
 #endif
 
-/* made backwards compatible -> now use mbox interface... */
+/* these are now backwards compatible -> they use mbox interface... */
 int pvm_delete		__ProtoGlarp__(( char *, int ));
 int pvm_insert		__ProtoGlarp__(( char *, int, int ));
 int pvm_lookup		__ProtoGlarp__(( char *, int, int * ));
