@@ -1,5 +1,5 @@
 
-/* $Id: Pvmfpack.c,v 1.2 1998/09/21 20:26:58 pvmsrc Exp $ */
+/* $Id: Pvmfpack.c,v 1.3 1999/12/10 15:22:14 pvmsrc Exp $ */
 
 #ifdef WIN32
 #include "..\..\include\pvm3.h"
@@ -35,6 +35,9 @@ __fortran PVMFPACK (what, vp, cnt, std, info)
 	case INTEGER4:
 		*info = pvm_pkint((int*)vp, *cnt, *std);
 		break;
+	case INTEGER8:
+		*info = pvm_pklong((long*)vp, *cnt, *std);
+		break;
 	case REAL4:
 		*info = pvm_pkfloat((float*)vp, *cnt, *std);
 		break;
