@@ -37,10 +37,6 @@
 #include "hostc.h"
 #include "bfunc.h"
 
-#ifndef	RSHCOMMAND
-#define	RSHCOMMAND	"/usr/ucb/rsh"
-#endif
-
 #ifndef	RSHTIMEOUT
 #define	RSHTIMEOUT	60
 #endif
@@ -187,6 +183,7 @@ static XrmOptionDescRec knownargs[] = {
 	{ "+mono", ".monochrome", XrmoptionNoArg, "off" },
 };
 
+char *pvmgetrsh();
 
 void pvm_cb();
 void quit_cb();
@@ -1095,7 +1092,7 @@ phase1(sp)
 			for (i = getdtablesize(); --i > 2; )
 				(void)close(i);
 			ac = 0;
-			av[ac++] = RSHCOMMAND;
+			av[ac++] = pvmgetrsh();
 			av[ac++] = hn;
 			if (hp->h_login) {
 				av[ac++] = "-l";
