@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: pvmdmimd.c,v 1.8 1999/07/08 19:00:27 kohl Exp $";
+	"$Id: pvmdmimd.c,v 1.10 2002/02/21 23:19:28 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -31,7 +31,7 @@ static char rcsid[] =
  */
 
 /*
- *	mpp.c
+ *	pvmdmimd.c
  *
  *  MPP interface.
  *
@@ -94,6 +94,20 @@ static char rcsid[] =
 /*
  *
  * $Log: pvmdmimd.c,v $
+ * Revision 1.10  2002/02/21 23:19:28  pvmsrc
+ * Added new (not to be documented!) PVM_MAX_TASKS env var support.
+ * 	- for Mahir Lokvancic <mahir@math.ufl.edu>.
+ * 	- forcefully limits the number of tasks that can attach to a
+ * 		pvmd, required on Solaris in rare circumstances when hard
+ * 		FD_SETSIZE limit is reached, and all hell breaks loose...
+ * 	- check return for task_new() call, can now produce NULL ptr,
+ * 		indicating PvmOutOfRes...
+ * (Spanker=kohl)
+ *
+ * Revision 1.9  2000/02/17 21:10:31  pvmsrc
+ * Cleaned up comments...  mpp_load() args...
+ * (Spanker=kohl)
+ *
  * Revision 1.8  1999/07/08 19:00:27  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -701,7 +715,11 @@ int trccod;				/* code to use on trace messages */
 		{
 			if (pids[j] > 0) 
 			{
-				tp = task_new(myhostpart + ptypepart + sp->n_first + j);
+				if ((tp = task_new(myhostpart + ptypepart
+						+ sp->n_first + j)) == NULL) {
+					err = PvmOutOfRes;
+					goto done;
+				}
 				task_setpid(tp, pids[j]);
 				tp->t_a_out = STRALLOC(name);
 				tp->t_ptid = ptid;
