@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: pvmdmimd.c,v 1.11 1999/07/08 19:00:28 kohl Exp $";
+	"$Id: pvmdmimd.c,v 1.14 2002/02/21 23:19:41 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -39,19 +39,19 @@ static char rcsid[] =
  *			Initialization. Create a table to keep track of active nodes.
  *			argc, argv: passed from main.
  *
- *		int mpp_load(int flags, char *name, char *argv, int count, int tids[],
- *					int ptid):
+ *		int mpp_load( struct waitc_spawn *wxp ) 
+ *
  *			Load executable onto nodes; create new entries in task table,
  *			encode node number and process type into task IDs, etc.
  *
  *				Construction of Task ID:
  *
- *				 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1
- *				+-+-+---------------------+-+-----------+---------------------+
- *				|s|g|		host index	  |n| instance  |	node # (2048)	  |
- *				+-+-+---------------------+-+-----------+---------------------+
+ *			 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
+ *			+-+-+-----------------------+-+-----+--------------------------+
+ *			|s|g|		host index	    |n| prt |    	node # (16384)	   |
+ *			+-+-+-----------------------+-+-----+--------------------------+
  *
- *				The "n" bit is set for node task but clear for host task.
+ *			The "n" bit is set for node task but clear for host task.
  *
  *			flags:	exec options;
  *			name:	executable to be loaded;
@@ -90,6 +90,29 @@ static char rcsid[] =
  *			tp: task pointer.
  *
  * $Log: pvmdmimd.c,v $
+ * Revision 1.14  2002/02/21 23:19:41  pvmsrc
+ * Added new (not to be documented!) PVM_MAX_TASKS env var support.
+ * 	- for Mahir Lokvancic <mahir@math.ufl.edu>.
+ * 	- forcefully limits the number of tasks that can attach to a
+ * 		pvmd, required on Solaris in rare circumstances when hard
+ * 		FD_SETSIZE limit is reached, and all hell breaks loose...
+ * 	- check return for task_new() call, can now produce NULL ptr,
+ * 		indicating PvmOutOfRes...
+ * (Spanker=kohl)
+ *
+ * Revision 1.13  2001/09/25 21:19:28  pvmsrc
+ * Minor TMPNAMFUN()/tmpnam() cleanup.
+ * 	- moved macro def to pvm3.h, renamed PVMTNPMAN().
+ * 	- same for LEN_OF_TMP_NAM -> PVMTMPNAMLEN.
+ * 	- mostly a huge waste of time, since *both* tmpnam() & mktemp()
+ * 		produce the same "dangerous" warning message in Linux/gcc...
+ * 	- damn.
+ * (Spanker=kohl)
+ *
+ * Revision 1.12  2000/02/17 21:10:44  pvmsrc
+ * Cleaned up comments...  mpp_load() args...
+ * (Spanker=kohl)
+ *
  * Revision 1.11  1999/07/08 19:00:28  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -217,7 +240,7 @@ static int myndf = 0;
 static struct nodeset *busynodes;	/* active nodes; ordered by proc type */
 static char pvmtxt[512];		/* scratch for error log */
 static int ptypemask;			/* mask; we use these bits of ptype in tids */
-static char nodefile[L_tmpnam];	/* tmp node file */
+static char nodefile[PVMTMPNAMLEN];	/* tmp node file */
 static char **nodelist = 0;		/* default poe node list */
 static int partsize = 0;		/* number of nodes allocated */
 static int hostfileused = TRUE;         /* Check if MP_HOSTFILE used */
@@ -332,7 +355,7 @@ sp2hostfile(first, count)
 		pvmlogperror(pvmtxt);
 		return (char *)0;
 	}
-	(void)tmpnam(nodefile);
+	(void)PVMTMPNAMFUN(nodefile);
 	if (!(tmpfp = fopen(nodefile, "w"))) {
 		sprintf(pvmtxt, "sp2hostfile() fopen %s", nodefile);
 		pvmlogperror(pvmtxt);
@@ -635,7 +658,10 @@ struct waitc_spawn *wxp;
 		/* create new task structs */
 
 		for (j = 0; j < count; j++) {
-			tp = task_new(myhostpart + ptypepart + j);
+			if ((tp = task_new(myhostpart + ptypepart + j)) == NULL) {
+				err = PvmOutOfRes;
+				goto done;
+			}
 			tp->t_a_out = STRALLOC(name);
 			tp->t_ptid = ptid;
 			tp->t_flag |= TF_CONN;		/* no need for the auth crap */
