@@ -1,5 +1,5 @@
 
-/* $Id: global.h,v 1.11 1999/07/08 19:00:20 kohl Exp $ */
+/* $Id: global.h,v 1.12 2001/02/07 23:14:03 pvmsrc Exp $ */
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -35,6 +35,10 @@
  *	Generic includes.
  *
  * $Log: global.h,v $
+ * Revision 1.12  2001/02/07 23:14:03  pvmsrc
+ * First Half of CYGWIN Check-ins...
+ * (Spanker=kohl)
+ *
  * Revision 1.11  1999/07/08 19:00:20  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -246,6 +250,9 @@
 #ifdef WIN32
 #define DEFBINDIR	"$PVM_ROOT\\bin\\$PVM_ARCH"
 #endif
+#ifdef CYGWIN
+#define	DEFBINDIR	"$HOME/pvm3/bin/$PVM_ARCH:$PVM_ROOT_U/bin/$PVM_ARCH"
+#endif
 
 #ifndef	DEFBINDIR
 #define	DEFBINDIR	"$HOME/pvm3/bin/$PVM_ARCH:$PVM_ROOT/bin/$PVM_ARCH"
@@ -255,12 +262,15 @@
 #define	DEFDEBUGGER	"$PVM_ROOT/lib/debugger"
 #endif
 
-#ifndef	PVMDFILE
 #ifdef IMA_OS2
 #define PVMDFILE	"lib/pvmd.cmd"
-#else
-#define	PVMDFILE	"lib/pvmd"
 #endif
+#ifdef CYGWIN
+#define	PVMDFILE	"lib/Win32/pvmd3.exe"
+#endif
+
+#ifndef	PVMDFILE
+#define	PVMDFILE	"lib/pvmd"
 #endif
 
 #ifndef	PVMDPATH
