@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: host.c,v 1.15 1999/07/08 18:59:52 kohl Exp $";
+	"$Id: host.c,v 1.21 2004/01/14 18:50:55 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -36,6 +36,35 @@ static char rcsid[] =
  *	Host table functions.
  *
  * $Log: host.c,v $
+ * Revision 1.21  2004/01/14 18:50:55  pvmsrc
+ * Added new AIX5* arches.
+ * (Spanker=kohl)
+ *
+ * Revision 1.20  2002/10/07 19:01:47  pvmsrc
+ * Bug in host table dumping - vmid can be NULL...
+ * 	- fix submitted by Kasidit Chanchio <chanchiok@ornl.gov>.
+ * (Spanker=kohl)
+ *
+ * Revision 1.19  2001/09/26 23:35:18  pvmsrc
+ * Added new hd_vmid to hostd struct.
+ * 	- use to override local PVM_VMID settings with hostfile "id=" option
+ * (Spanker=kohl)
+ *
+ * Revision 1.18  2001/02/07 23:14:03  pvmsrc
+ * First Half of CYGWIN Check-ins...
+ * (Spanker=kohl)
+ *
+ * Revision 1.17  2000/07/12 19:24:09  pvmsrc
+ * Goofy fix for FreeBSD - variation on ifreq struct.
+ * 	- submitted via Frederik Meerwaldt <frederik@freddym.org>.
+ * (Spanker=kohl)
+ *
+ * Revision 1.16  2000/02/16 21:59:39  pvmsrc
+ * Fixed up #include <sys/types.h> stuff...
+ * 	- use <bsd/sys/types.h> for IMA_TITN...
+ * 	- #include before any NEEDMENDIAN #includes...
+ * (Spanker=kohl)
+ *
  * Revision 1.15  1999/07/08 18:59:52  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -134,6 +163,13 @@ static char rcsid[] =
 #ifndef WIN32
 #include <sys/param.h>
 #endif
+
+#ifdef IMA_TITN
+#include <bsd/sys/types.h>
+#else
+#include <sys/types.h>
+#endif
+
 #ifdef NEEDMENDIAN
 #include <machine/endian.h>
 #endif
@@ -145,21 +181,21 @@ static char rcsid[] =
 #endif
 
 #include <pvm3.h>
-#ifdef WIN32
-#include "pvmwin.h"
-#ifndef _XDR_HEADERS
-#define _XDR_HEADERS
+
+#if defined(WIN32) || defined(CYGWIN)
 #include "..\xdr\types.h"
 #include "..\xdr\xdr.h"
-#endif
 #else
 #include <rpc/types.h>
 #include <rpc/xdr.h>
+#endif
+
+#ifdef WIN32
+#include "pvmwin.h"
+#else
 #ifndef IMA_TITN
-#include <sys/types.h>
 #include <sys/ioctl.h>
 #else
-#include <bsd/sys/types.h>
 #include <sys/43ioctl.h>
 #endif
 
@@ -296,6 +332,8 @@ hd_free(hp)
 	}
 	if (hp->hd_aname)
 		PVM_FREE(hp->hd_aname);
+	if (hp->hd_vmid)
+		PVM_FREE(hp->hd_vmid);
 	PVM_FREE(hp);
 }
 
@@ -353,9 +391,10 @@ hd_dump(hp)
 			n);
 
 	pvmlogprintf(
-			"           tx %d rx %d rtt %d.%06d\n",
+			"           tx %d rx %d rtt %d.%06d id \"%s\"\n",
 			hp->hd_txseq, hp->hd_rxseq,
-			hp->hd_rtt.tv_sec, hp->hd_rtt.tv_usec);
+			hp->hd_rtt.tv_sec, hp->hd_rtt.tv_usec,
+			(hp->hd_vmid ? hp->hd_vmid : ""));
 }
 
 
@@ -663,6 +702,9 @@ applydefaults(hp, defhp)
 	if (!(hp->hd_flag & HF_SPEED) && (defhp->hd_flag & HF_SPEED))
 		hp->hd_speed = defhp->hd_speed;
 
+	if (!hp->hd_vmid && defhp->hd_vmid)
+		hp->hd_vmid = STRALLOC(defhp->hd_vmid);
+
 	hp->hd_flag |= defhp->hd_flag;
 
 	return 0;
@@ -741,6 +783,12 @@ parsehost(buf, hp)
 			hp->hd_aname = STRALLOC(av[ac] + 3);
 			continue;
 		}
+		if (!strncmp(av[ac], "id=", 3)) {
+			if (hp->hd_vmid)
+				PVM_FREE(hp->hd_vmid);
+			hp->hd_vmid = STRALLOC(av[ac] + 3);
+			continue;
+		}
 		pvmlogprintf("parsehost(): unknown option \"%s\"\n", av[ac]);
 		err++;
 	}
@@ -928,7 +976,9 @@ iflist(alp, np)
 	sif.ifc_len = sizeof(buf);
 	if (ioctl(soc,
 #ifdef OSIOCGIFCONF
-#if defined(IMA_RS6K) || defined(IMA_RS6KMP) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_FREEBSD)
+#if defined(IMA_RS6K) || defined(IMA_RS6KMP) || defined(IMA_SP2MPI) \
+			|| defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2) \
+			|| defined(IMA_FREEBSD)
 		SIOCGIFCONF
 #else
 		OSIOCGIFCONF
@@ -953,6 +1003,8 @@ iflist(alp, np)
 			perror("ioctl");
 			goto bail;
 		}
+		/* On some FreeBSD systems: */
+		/* if (IFF_UP & req.ifr_flags) { */
 		if (IFF_UP & req.ifr_ifru.ifru_flags) {
 			if (nip > 0 && !(nip % 10))
 				iplist = TREALLOC(iplist, (nip + 10), struct in_addr);
