@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: hoster.c,v 1.15 1999/08/19 15:39:24 pvmsrc Exp $";
+	"$Id: hoster.c,v 1.20 2001/09/27 21:25:09 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -36,6 +36,36 @@ static char rcsid[] =
  *	Slave pvmd starter.
  *
  * $Log: hoster.c,v $
+ * Revision 1.20  2001/09/27 21:25:09  pvmsrc
+ * BEOSCYLD port.
+ * 	- submitted by Joe Vitale <vitale@scyld.com>.
+ * 	(we renamed it from BEOWULF to BEOSCYLD, but it's his port... :-)
+ * (Spanker=kohl)
+ *
+ * Revision 1.19  2001/09/26 21:22:17  pvmsrc
+ * Added Handling for Optional Virtual Machine ID.
+ * 	- extra vmid comes through with SM_STHOST message (after wincmd).
+ * 	- instruct user to type VMID on remote pvmd stdin for manual startup
+ * 	- in phase1(), after potential rsh/rexec, write VMID env string
+ * 		to remote pvmd's stdin.
+ * (Spanker=kohl)
+ *
+ * Revision 1.18  2001/02/07 23:14:04  pvmsrc
+ * First Half of CYGWIN Check-ins...
+ * (Spanker=kohl)
+ *
+ * Revision 1.17  2000/02/16 21:59:40  pvmsrc
+ * Fixed up #include <sys/types.h> stuff...
+ * 	- use <bsd/sys/types.h> for IMA_TITN...
+ * 	- #include before any NEEDMENDIAN #includes...
+ * (Spanker=kohl)
+ *
+ * Revision 1.16  2000/02/14 20:31:57  pvmsrc
+ * Lose #define-d RSHCOMMAND usage.
+ * 	- use new pvmgetrsh() routine, which checks for PVM_RSH or else
+ * 		uses old RSHCOMMAND interface.
+ * (Spanker=kohl)
+ *
  * Revision 1.15  1999/08/19 15:39:24  pvmsrc
  * Whoa...  New wincmd stuff was whacking addhost protocol!
  * 	- *always* pack in something for wincmd, else the attempt to unpack
@@ -169,6 +199,12 @@ static char rcsid[] =
  *
  */
 
+#ifdef IMA_TITN
+#include <bsd/sys/types.h>
+#else
+#include <sys/types.h>
+#endif
+
 #ifdef NEEDMENDIAN
 #include <machine/endian.h>
 #endif
@@ -180,25 +216,20 @@ static char rcsid[] =
 #endif
 
 #include <pvm3.h>
-#ifdef WIN32
-#ifndef _XDR_HEADERS
-#define _XDR_HEADERS
+
+#if defined(WIN32) || defined(CYGWIN)
 #include "..\xdr\types.h"
 #include "..\xdr\xdr.h"
-#endif
 #else
 #include <rpc/types.h>
 #include <rpc/xdr.h>
-#include <sys/time.h>
-#include <sys/socket.h>
-#include <netinet/in.h>                                                  
-#include <netdb.h>
 #endif
 
-#ifdef IMA_TITN
-#include <bsd/sys/types.h>
-#else
-#include <sys/types.h>
+#ifndef WIN32
+#include <sys/time.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <netdb.h>
 #endif
 
 #ifdef NEEDSSELECTH
@@ -226,8 +257,8 @@ static char rcsid[] =
 #include "bfunc.h"
 #include "global.h"
 
-#ifndef	RSHCOMMAND
-#define	RSHCOMMAND	"/usr/ucb/rsh"
+#ifdef IMA_BEOSCYLD
+#include <string.h>
 #endif
 
 #ifndef	RSHTIMEOUT
@@ -257,6 +288,7 @@ struct hst {
 #define	HST_MANUAL		2		/* do manual startup */
 	char *h_cmd;
 	char *h_wincmd;				/* alternate WIN32 default pvmd cmd */
+	char *h_vmid;				/* optional virtual machine ID */
 	char *h_result;
 };
 
@@ -286,6 +318,8 @@ extern void pvmbailout();
 extern int pvmdebmask;				/* from pvmd.c */
 extern char *username;				/* from pvmd.c */
 
+char *pvmgetrsh();
+
 
 /***************
  **  Private  **
@@ -326,7 +360,8 @@ hoster(mp)
 			|| upkstralloc(mp, &hp->h_sopts)
 			|| upkstralloc(mp, &hp->h_login)
 			|| upkstralloc(mp, &hp->h_cmd)
-			|| upkstralloc(mp, &hp->h_wincmd)) {
+			|| upkstralloc(mp, &hp->h_wincmd)
+			|| upkstralloc(mp, &hp->h_vmid)) {
 				pvmlogerror("hoster() bad message format\n");
 				pvmbailout(0);
 			}
@@ -335,6 +370,11 @@ hoster(mp)
 				PVM_FREE(hp->h_wincmd);
 				hp->h_wincmd = 0;
 			}
+			/* Check for (optional) virtual machine ID */
+			if (!strcmp(hp->h_vmid,"")) {
+				PVM_FREE(hp->h_vmid);
+				hp->h_vmid = 0;
+			}
 			if (pvmdebmask & PDMSTARTUP) {
 				pvmlogprintf("%d. t%x %s so=\"%s\"\n", i,
 						hp->h_tid,
@@ -718,7 +758,7 @@ phase1(sp)
 {
 	struct hst *hp;
 	char *hn;
-	char *av[16];			/* for rsh args */
+	char *av[32];			/* for rsh args */
 	int ac;
 	char buf[512];
 	int pid = -1;			/* pid of rsh */
@@ -771,6 +811,17 @@ phase1(sp)
 		if (*p == '\n')
 			*p = 0;
 		hp->h_result = STRALLOC(buf);
+
+	/* send vmid, if set */
+
+		if (hp->h_vmid) {
+			fprintf(stderr, "Now Type the Virtual Machine ID on %s:\n",
+					hn);
+			fprintf(stderr, "%s\n", hp->h_vmid);
+		}
+
+	/* done */
+
 		fprintf(stderr, "Thanks\n");
 		fflush(stderr);
 		return 0;
@@ -820,13 +871,33 @@ phase1(sp)
 			for (i = getdtablesize(); --i > 2; )
 				(void)close(i);
 			ac = 0;
-			av[ac++] = RSHCOMMAND;
+			av[ac++] = pvmgetrsh();
 			av[ac++] = hn;
 			if (hp->h_login) {
 				av[ac++] = "-l";
 				av[ac++] = hp->h_login;
 			}
+#ifdef IMA_BEOSCYLD
+			/* Chop up the command line and all its parameters into
+			 * individual strings. The underlying assumption here is
+			 * that PVM_RSH is defined to specify 'bpsh' instead of
+			 * 'rsh/ssh'. This bit of code converts the single string
+			 * av[2] from this: "pvmd3 -option1 -option2", into this:
+			 * av[2] = "pvmd3", av[3] = "-option1", av[4] = "-option2").
+			 * If this is not done 'bpsh' treats the original av[2] as
+			 * the file name of the command to be executed, which will
+			 * fail to execute.
+			 */
+			p = hp->h_cmd;
+			do {
+				av[ac++] = p;
+				p = strstr(p," ");
+				if (p != NULL)
+					*p++ = '\0';
+			} while (p != NULL);
+#else
 			av[ac++] = hp->h_cmd;
+#endif
 			av[ac++] = 0;
 			if (pvmdebmask & PDMSTARTUP) {
 				for (ac = 0; av[ac]; ac++)
@@ -863,6 +934,21 @@ phase1(sp)
 		}
 #endif
 	}
+
+	/* send vmid, if set */
+	if (sp->s_hst->h_vmid && sp->s_wfd >= 0) {
+#ifdef WIN32
+		win32_write_socket(sp->s_wfd, "PVM_VMID=", 9);
+		win32_write_socket(sp->s_wfd,
+				sp->s_hst->h_vmid, strlen(sp->s_hst->h_vmid));
+		win32_write_socket(sp->s_wfd, "\n", 1);
+#else
+		write(sp->s_wfd, "PVM_VMID=", 9);
+		write(sp->s_wfd, sp->s_hst->h_vmid, strlen(sp->s_hst->h_vmid));
+		write(sp->s_wfd, "\n", 1);
+#endif
+	}
+
 	return 0;
 
 oops:
