@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: lpvm.c,v 1.53 1999/07/08 18:59:53 kohl Exp $";
+	"$Id: lpvm.c,v 1.69 2004/01/14 18:50:55 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -36,6 +36,99 @@ static char rcsid[] =
  *	Libpvm core for unix environment.
  *
  * $Log: lpvm.c,v $
+ * Revision 1.69  2004/01/14 18:50:55  pvmsrc
+ * Added new AIX5* arches.
+ * (Spanker=kohl)
+ *
+ * Revision 1.68  2003/02/04 20:03:04  pvmsrc
+ * Got rid of Jigen's goofy my_errno...
+ * 	- never resolved anywhere...
+ * (Spanker=kohl)
+ *
+ * Revision 1.67  2002/04/16 15:06:01  pvmsrc
+ * WIN32 Fixes for Multiple Domains.
+ * 	- submitted by Jigen Zhou <jigen@icemcfd.com>.
+ * (Spanker=kohl)
+ *
+ * Revision 1.66  2002/02/18 19:18:33  pvmsrc
+ * Added "|| defined(IMA_SUN4SOL2)" to "const void *i, *j" #if
+ * 	for int_compare() / qsort()...
+ * 	(needed for sure for 64 bit) Solaris)
+ * (Spanker=kohl)
+ *
+ * Revision 1.65  2001/12/07 16:06:09  pvmsrc
+ * Oops!  Some new Unix fixes & features break Windows (of course!).
+ * 	- #ifdef-ed away fcntl() calls for non-blocking sockets / stdin.
+ * 	(one for bug fix in lpvm.c, one for check_ext_input() in pvmd.c)
+ * (Spanker=kohl)
+ *
+ * Revision 1.64  2001/09/25 21:20:17  pvmsrc
+ * Minor TMPNAMFUN()/tmpnam() cleanup.
+ * 	- moved macro def to pvm3.h, renamed PVMTNPMAN().
+ * 	- same for LEN_OF_TMP_NAM -> PVMTMPNAMLEN.
+ * 	- mostly a huge waste of time, since *both* tmpnam() & mktemp()
+ * 		produce the same "dangerous" warning message in Linux/gcc...
+ * 	- damn.
+ * (Spanker=kohl)
+ *
+ * Revision 1.63  2001/09/25 17:28:09  pvmsrc
+ * Fixed leftover tmpnam() usage -> use TMPNAMFUN() instead...
+ * 	- also got rid of "double" call to tmpnam()...
+ * (Spanker=kohl)
+ *
+ * Revision 1.62  2001/05/11 19:40:50  pvmsrc
+ * Fixed direct-routing bug, removed blocking socket deadlock.
+ * 	- fix submitted by Kamil Iskra <kamil@wins.uva.nl> (as well as
+ * 		<jc@esi.fr>, Dick van Albada <dick@wins.uva.nl>, and
+ * 		Zeger Hendrikse <zegerh@wins.uva.nl>).
+ * (Spanker=kohl)
+ *
+ * Revision 1.61  2001/05/11 19:21:29  pvmsrc
+ * Added "&& errno != EAGAIN" to select() return code check in mxfer().
+ * 	- reported by "Johannes Hennecke" <Johannes.Hennecke@ELSA.de>
+ * (Spanker=kohl)
+ *
+ * Revision 1.60  2001/05/11 17:32:26  pvmsrc
+ * Eliminated references to sys_errlist & sys_nerr.
+ * 	- unnecessary, and we're whacking that crap anyway.
+ * (Spanker=kohl)
+ *
+ * Revision 1.59  2001/02/07 23:14:04  pvmsrc
+ * First Half of CYGWIN Check-ins...
+ * (Spanker=kohl)
+ *
+ * Revision 1.58  2000/06/15 17:51:49  pvmsrc
+ * Fixed bug in WIN32 direct routing.
+ * 	- stupid #endif in the wrong place, pvm_fd_add() call whacked.
+ * 	- turned back on direct routing default and setopt.
+ * (Spanker=kohl)
+ *
+ * Revision 1.57  2000/02/17 23:12:10  pvmsrc
+ * *** Changes for new BEOLIN port ***
+ * 	- MPP-like, similar to SP2, etc.
+ * 	- submitted by Paul Springer <pls@smokeymt.jpl.nasa.gov>.
+ * 	- format-checked & cleaned up by Jeembo...  :-)
+ * (Spanker=kohl)
+ *
+ * Revision 1.56  2000/02/16 21:59:40  pvmsrc
+ * Fixed up #include <sys/types.h> stuff...
+ * 	- use <bsd/sys/types.h> for IMA_TITN...
+ * 	- #include before any NEEDMENDIAN #includes...
+ * (Spanker=kohl)
+ *
+ * Revision 1.55  2000/02/07 22:22:07  pvmsrc
+ * Hack to help with select()/fd_sets in WIN32:
+ * 	- fd_set is *NOT* a bit field in WIN32, is a circular buffer.
+ * 	- must check for !FD_ISSET() before FD_SET() else duplicate
+ * 		entries possible, and fd_set buffer overflow/overwrite.
+ * 	- similary, FD_ISSET() check before FD_CLR().
+ * 	- patch submitted by "Bruce W. Church" <bwc1@cornell.edu>.
+ * (Spanker=kohl)
+ *
+ * Revision 1.54  1999/11/08 17:44:29  pvmsrc
+ * SGI compiler cleanup.
+ * (Spanker=kohl)
+ *
  * Revision 1.53  1999/07/08 18:59:53  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -467,10 +560,16 @@ static char rcsid[] =
 #include <process.h>
 #include <stdlib.h>
 #include <search.h>
-#include <sys/types.h>
 #endif
 
 #include <stdio.h>
+
+#ifdef IMA_TITN
+#include <bsd/sys/types.h>
+#else
+#include <sys/types.h>
+#endif
+
 #ifdef NEEDMENDIAN
 #include <machine/endian.h>
 #endif
@@ -482,15 +581,16 @@ static char rcsid[] =
 #endif
 
 #include <pvm3.h>
-#ifdef WIN32
-#ifndef _XDR_HEADERS
-#define _XDR_HEADERS
+
+#if defined(WIN32) || defined(CYGWIN)
 #include "..\xdr\types.h"
 #include "..\xdr\xdr.h"
-#endif
 #else
 #include <rpc/types.h>
 #include <rpc/xdr.h>
+#endif
+
+#ifndef WIN32
 #include <sys/socket.h>
 #endif
 
@@ -509,6 +609,11 @@ static char rcsid[] =
 #endif
 #include <errno.h>
 #include <signal.h>
+
+#ifdef IMA_BEOLIN
+#include <netdb.h>
+#endif
+
 #include <pvmproto.h>
 #include "pvmalloc.h"
 #include "pvmfrag.h"
@@ -547,20 +652,6 @@ static char rcsid[] =
 #define	TTSOCKBUF	0x8000
 #endif
 
-#ifdef	NOTMPNAM
-#define	TMPNAMFUN(x)	pvmtmpnam(x)
-#define	LEN_OF_TMP_NAM	32
-char *pvmtmpnam();
-
-#else	/*NOTMPNAM*/
-#define	TMPNAMFUN(x)	tmpnam(x)
-#ifdef	L_tmpnam
-#define	LEN_OF_TMP_NAM	L_tmpnam
-#else
-#define	LEN_OF_TMP_NAM	64
-#endif
-#endif	/*NOTMPNAM*/
-
 
 /***************
  **  Globals  **
@@ -582,8 +673,6 @@ struct msgid *pvm_inprecv = (struct msgid *) NULL;
 
 #ifndef HASERRORVARS
 extern int errno;						/* from libc */
-extern char *sys_errlist[];
-extern int sys_nerr;
 #endif
 
 #if defined(IMA_PGON)
@@ -637,24 +726,43 @@ pvm_fd_add(fd, sets)
 	int sets;			/* which sets */
 {
 #ifdef	SANITY
+#ifndef WIN32
 	if (fd < 0 || fd >= FD_SETSIZE) {
 		pvmlogprintf("pvm_fd_add() bad fd %d\n", fd);
 		return 1;
 	}
+#endif
 #endif
 	if (sets & 1)
-		FD_SET(fd, &pvmrfds);
+		if ( !FD_ISSET(fd, &pvmrfds) ) {
+			FD_SET(fd, &pvmrfds);
+#ifdef WIN32
+			pvmnfds++;
+#endif
+		}
 /*
 	if (sets & 2)
-		FD_SET(fd, &pvmwfds);
+		if ( !FD_ISSET(fd, &pvmwfds) ) {
+			FD_SET(fd, &pvmwfds);
+#ifdef WIN32
+			pvmnwfds++;
+#endif
+		}
 	if (sets & 4)
-		FD_SET(fd, &pvmefds);
+		if ( !FD_ISSET(fd, &pvmefds) ) {
+			FD_SET(fd, &pvmefds);
+#ifdef WIN32
+			pvmnefds++;
+#endif
+		}
 */
 
+#ifndef WIN32
 	/* if this is new highest, adjust nfds */
-
 	if (fd >= pvmnfds)
 		pvmnfds = fd + 1;
+#endif
+
 	return 0;
 }
 
@@ -664,35 +772,54 @@ pvm_fd_delete(fd, sets)
 	int sets;			/* which sets */
 {
 #ifdef	SANITY
+#ifndef WIN32
 	if (fd < 0 || fd >= FD_SETSIZE) {
 		pvmlogprintf("pvm_fd_delete() bad fd %d\n", fd);
 		return 1;
 	}
+#endif
 #endif
 	if (sets & 1)
-		FD_CLR(fd, &pvmrfds);
+		if ( FD_ISSET(fd, &pvmrfds) ) {
+			FD_CLR(fd, &pvmrfds);
+#ifdef WIN32
+			pvmnfds--;
+#endif
+		}
 /*
 	if (sets & 2)
-		FD_CLR(fd, &pvmwfds);
+		if ( FD_ISSET(fd, &pvmwfds) ) {
+			FD_CLR(fd, &pvmwfds);
+#ifdef WIN32
+			pvmnwfds--;
+#endif
+		}
 	if (sets & 4)
-		FD_CLR(fd, &pvmefds);
+		if ( FD_ISSET(fd, &pvmefds) ) {
+			FD_CLR(fd, &pvmefds);
+#ifdef WIN32
+			pvmnefds--;
+#endif
+		}
 */
 
+#ifndef WIN32
 	/* if this was highest, may have to adjust nfds to new highest */
-
 	if (fd + 1 == pvmnfds)
 		while (pvmnfds > 0) {
 			pvmnfds--;
 			if (FD_ISSET(pvmnfds, &pvmrfds)
 /*
-			|| FD_ISSET(pvmnfds, &pvmefds)
-			|| FD_ISSET(pvmnfds, &pvmwfds)
+			|| FD_ISSET(pvmnefds, &pvmefds)
+			|| FD_ISSET(pvmnwfds, &pvmwfds)
 */
 			) {
 				pvmnfds++;
 				break;
 			}
 		}
+#endif
+
 	return 0;
 }
 
@@ -990,7 +1117,7 @@ pvm_tc_conreq(mid)
 	int i;
 	int ictx;
 #ifdef SOCKLENISUINT
-#ifdef IMA_AIX4SP2
+#if defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	unsigned int oslen;
 #else
 	size_t oslen;
@@ -1000,7 +1127,7 @@ pvm_tc_conreq(mid)
 #endif
 #ifndef NOUNIXDOM
 	struct sockaddr_un uns;
-	char spath[LEN_OF_TMP_NAM];
+	char spath[PVMTMPNAMLEN];
 #endif
 	char buf[256];
 
@@ -1091,11 +1218,10 @@ pvm_tc_conreq(mid)
 					pvmlogperror("pvm_tc_conreq() socket");
 
 				} else {
-					(void)TMPNAMFUN(spath);
 					BZERO((char*)&uns, sizeof(uns));
 					uns.sun_family = AF_UNIX;
 					spath[0] = 0;
-					(void)tmpnam(spath);
+					(void)PVMTMPNAMFUN(spath);
 					strcpy(uns.sun_path, spath);
 
 					oslen = sizeof(uns);
@@ -1320,8 +1446,8 @@ pvm_tc_conack(mid)
 #endif
 								(void)fcntl(pcbp->tt_fd, F_SETFL, i);
 							}
-							pvm_fd_add(pcbp->tt_fd, 1);
 #endif
+							pvm_fd_add(pcbp->tt_fd, 1);
 						}
 					}
 				}
@@ -1402,7 +1528,7 @@ mxinput(pcbp, nfr)
 	int n;					/* bytes received */
 	int m;					/* length of fragment */
 	struct pmsg *rxup;		/* message containing this frag */
-	struct pmsg *frgpile, *hdfrgpile;
+	struct pmsg *hdfrgpile;
 	char *cp;				/* gp */
 	int src;
 	int dst;
@@ -1540,9 +1666,9 @@ mxinput(pcbp, nfr)
 		return gotem;
 	}
 
-	frgpile = hdfrgpile = pvm_mpp_pmsgs();
+	hdfrgpile = pvm_mpp_pmsgs();
 #else
-	frgpile = hdfrgpile = pcbp->tt_rxfrag ;
+	hdfrgpile = pcbp->tt_rxfrag ;
 #endif
 	/*
 	* if start of message, make new umbuf, add to frag pile
@@ -1655,7 +1781,7 @@ mxfer(txup, tmout)
 	int ff;
 	int n;
 #ifdef SOCKLENISUINT
-#ifdef IMA_AIX4SP2
+#if defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	unsigned int oslen;
 #else
 	size_t oslen;
@@ -1814,7 +1940,11 @@ mxfer(txup, tmout)
 				(fd_set *)&rfds, (fd_set *)&wfds, (fd_set*)0,
 #endif
 				tvp)) == -1
-		&& errno != EINTR) {
+		&& errno != EINTR
+#ifdef IMA_LINUX
+		&& errno != EAGAIN
+#endif
+		) {
 			pvmlogperror("mxfer() select");
 			return PvmSysErr;
 		}
@@ -1881,13 +2011,16 @@ mxfer(txup, tmout)
 							if (setsockopt(s, IPPROTO_TCP, TCP_NODELAY,
 									(char*)&n, sizeof(int)) == -1)
 								pvmlogperror("mxfer() setsockopt");
-#if (!defined(IMA_RS6K)) && (!defined(IMA_AIX46K))
+#if (!defined(IMA_RS6K)) \
+		&& (!defined(IMA_AIX46K)) && (!defined(IMA_AIX46K64)) \
+		&& (!defined(IMA_AIX56K)) && (!defined(IMA_AIX56K64))
 							n = TTSOCKBUF;
 							if (setsockopt(s, SOL_SOCKET, SO_SNDBUF,
 									(char*)&n, sizeof(int)) == -1
 							|| setsockopt(s, SOL_SOCKET, SO_RCVBUF,
 									(char*)&n, sizeof(n)) == -1)
-								pvmlogperror("mxfer() setsockopt SO_SNDBUF");
+								pvmlogperror(
+									"mxfer() setsockopt SO_SNDBUF");
 #endif
 #ifndef NOUNIXDOM
 						}
@@ -1909,6 +2042,25 @@ mxfer(txup, tmout)
 						check_routedelete(pcbp);
 						pcbp->tt_fd = s;
 						pcbp->tt_state = TTOPEN;
+
+#ifndef WIN32
+						{
+							int i;
+
+							if ((i = fcntl(pcbp->tt_fd, F_GETFL, 0))
+									== -1)
+								pvmlogperror("mxfer() fcntl");
+							else {
+#ifdef O_NDELAY
+								i |= O_NDELAY;
+#else
+								i |= FNDELAY;
+#endif
+								(void)fcntl(pcbp->tt_fd, F_SETFL, i);
+							}
+						}
+#endif
+
 						pvm_fd_add(s, 1);
 
 	/* new route socket created for communication */
@@ -1918,6 +2070,8 @@ mxfer(txup, tmout)
 				}
 		}
 
+		bypassRead = bypassRead; /* sgi compiler */
+
 #else	/* !IMA_MPP */
 
 		/* For Native Messaging mxinput is used to probe pvmd AND tasks when 
@@ -2052,8 +2206,10 @@ mxfer(txup, tmout)
 				n = pvm_node_send(txcp, txtogo, txpcbp, &sendmsg, 
 							inPlaceHeader, inPlaceBodyLen);
 #else 
+				inPlaceBodyLen = inPlaceBodyLen; /* sgi compiler */
 
-#if defined(IMA_RS6K) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_RS6K) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) \
+		|| defined(IMA_AIX5SP2)
 				n = write(txpcbp->tt_fd, txcp, min(txtogo, 4096));
 #else
 #ifndef WIN32
@@ -2197,7 +2353,7 @@ mroute(mid, dtid, tag, tmout)
 	struct sockaddr_in sad;
 	int l;
 #ifdef SOCKLENISUINT
-#ifdef IMA_AIX4SP2
+#if defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	unsigned int oslen;
 #else
 	size_t oslen;
@@ -2211,7 +2367,7 @@ mroute(mid, dtid, tag, tmout)
 	char *addr = 0;
 #ifndef NOUNIXDOM
 	struct sockaddr_un uns;
-	char spath[LEN_OF_TMP_NAM];
+	char spath[PVMTMPNAMLEN];
 #endif
 
 	if (up = midtobuf(mid)) {
@@ -2278,11 +2434,10 @@ mroute(mid, dtid, tag, tmout)
 					pvmlogperror("mroute() socket");
 
 				} else {
-					(void)TMPNAMFUN(spath);
 					BZERO((char*)&uns, sizeof(uns));
 					uns.sun_family = AF_UNIX;
 					spath[0] = 0;
-					(void)tmpnam(spath);
+					(void)PVMTMPNAMFUN(spath);
 					strcpy(uns.sun_path, spath);
 
 					oslen = sizeof(uns);
@@ -2326,7 +2481,9 @@ mroute(mid, dtid, tag, tmout)
 							if (setsockopt(s, IPPROTO_TCP, TCP_NODELAY,
 									(char*)&l, sizeof(int)) == -1)
 								pvmlogperror("mroute() setsockopt");
-#if (!defined(IMA_RS6K)) && (!defined(IMA_AIX46K))
+#if (!defined(IMA_RS6K)) \
+		&& (!defined(IMA_AIX46K)) && (!defined(IMA_AIX46K64)) \
+		&& (!defined(IMA_AIX56K)) && (!defined(IMA_AIX56K64))
 							l = TTSOCKBUF;
 							if (setsockopt(s, SOL_SOCKET, SO_SNDBUF,
 									(char*)&l, sizeof(l)) == -1
@@ -2504,7 +2661,7 @@ msendrecv(other, tag, context)
 
 static int
 int_compare(i, j)
-#if defined (WIN32) || defined(IMA_SGI) || defined(IMA_SGI5) || defined(IMA_SGI6) || defined(IMA_SGI64) || defined(IMA_SGIMP) || defined(IMA_SGIMP6) || defined(IMA_SGIMP64)
+#if defined (WIN32) || defined(IMA_SGI) || defined(IMA_SGI5) || defined(IMA_SGI6) || defined(IMA_SGI64) || defined(IMA_SGIMP) || defined(IMA_SGIMP6) || defined(IMA_SGIMP64) || defined(IMA_SUN4SOL2)
 	const void *i, *j;
 #else
 	void *i, *j;
@@ -2618,7 +2775,7 @@ mksocs()
 #endif
 
 #ifdef SOCKLENISUINT
-#ifdef IMA_AIX4SP2
+#if defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	unsigned int oslen;
 #else
 	size_t oslen;
@@ -2822,7 +2979,7 @@ pvmbeatask()
 	int cookie;						/* cookie assigned by pvmd for ident */
 	int cc;
 	char **ep=0;
-	char authfn[LEN_OF_TMP_NAM];	/* auth file name */
+	char authfn[PVMTMPNAMLEN];	/* auth file name */
 	int authfd = -1;				/* auth fd to validate pvmd ident */
 	int i;
 	char buf[16];					/* for converting sockaddr */
@@ -2835,6 +2992,10 @@ pvmbeatask()
 	char tmask[ 2 * TEV_MASK_LENGTH ];
 	int tbuf, topt;
 	int mid;
+#ifdef IMA_BEOLIN
+	struct hostent *hostaddr;
+	char namebuf[128];
+#endif
 	TEV_DECLS
 #ifdef WIN32
 
@@ -2926,7 +3087,8 @@ pvmbeatask()
 	*/
 
 #ifndef IMA_MPP
-	(void)TMPNAMFUN(authfn);
+#ifndef NOPROT
+	(void)PVMTMPNAMFUN(authfn);
 #ifdef IMA_OS2
 	if ((authfd = open(authfn, O_RDWR|O_CREAT|O_TRUNC, 0600)) == -1) {
 #else
@@ -2937,6 +3099,7 @@ pvmbeatask()
 		cc = PvmSysErr;
 		goto bail2;
 	}
+#endif
 
 	/*
 	*	send first connect message to pvmd
@@ -2965,6 +3128,12 @@ pvmbeatask()
 	*	check our t-auth file; write in pvmd d-auth file
 	*/
 
+#ifndef NOPROT
+
+#ifdef IMA_BEOLIN
+	sleep(15);  /* because of race condition over NFS--yuck! (PLS) */
+#endif
+
 	if ((cc = read(authfd, (char*)&cc, 1)) == -1) {
 		pvmlogperror("pvmbeatask() read authfile");
 		cc = PvmSysErr;
@@ -2999,6 +3168,8 @@ pvmbeatask()
 	authfd = -1;
 	authfn[0] = 0;
 
+#endif
+
 	/*
 	*	send second connect message to pvmd
 	*/
@@ -3105,7 +3276,28 @@ pvmbeatask()
 	pvm_upkint(&i, 1, 1);	/* XXX data signature */
 
 	pvm_upkstr(buf);
+
+#ifdef IMA_BEOLIN
+	if (gethostname(namebuf, sizeof(namebuf)-1) == -1) {
+		pvmlogerror("pvmbeatask() can't gethostname()\n");
+		hex_inadport(buf, &pvmourinet);
+	} else {
+		/* got name, now get addr */
+		if (!(hostaddr = gethostbyname( namebuf ))) {
+			pvmlogprintf( "pvmbeatask() can't gethostbyname() for %s\n",
+					namebuf );
+			hex_inadport(buf, &pvmourinet);
+		} else {
+			/* got addr, now save it */
+			BCOPY( hostaddr->h_addr_list[0],
+					(char*)&pvmourinet.sin_addr,
+					sizeof(struct in_addr));
+		}
+	}
+#else
 	hex_inadport(buf, &pvmourinet);
+#endif
+
 	pvmourinet.sin_family = AF_INET;
 	pvmourinet.sin_port = 0;
 
