@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: lpvmmimd.c,v 1.23 1999/07/08 18:59:59 kohl Exp $";
+	"$Id: lpvmmimd.c,v 1.25 2004/01/14 18:50:57 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -36,6 +36,15 @@ static char rcsid[] =
  *	Libpvm core for MPP environment.
  *
  * $Log: lpvmmimd.c,v $
+ * Revision 1.25  2004/01/14 18:50:57  pvmsrc
+ * Added new AIX5* arches.
+ * (Spanker=kohl)
+ *
+ * Revision 1.24  2001/05/11 17:32:28  pvmsrc
+ * Eliminated references to sys_errlist & sys_nerr.
+ * 	- unnecessary, and we're whacking that crap anyway.
+ * (Spanker=kohl)
+ *
  * Revision 1.23  1999/07/08 18:59:59  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -258,7 +267,7 @@ static char rcsid[] =
 #ifdef IMA_I860
 #include <cube.h>
 #endif
-#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 #include <sys/socket.h>
 #include <sys/select.h>
 #include "mpi.h"
@@ -294,8 +303,6 @@ extern struct encvec *enctovec();
 
 #ifndef HASERRORVARS
 extern int errno;					/* from libc */
-extern char *sys_errlist[];
-extern int sys_nerr;
 #endif
 
 int pvmtidhmask = TIDHOST;			/* mask - host field of tids */
@@ -316,7 +323,8 @@ static struct pmsg *rxfrag = 0;			/* not-assembled incm msgs */
 static int mpierrcode = 0;				/* error code returned by MPI calls */
 static struct tmpfrag outfrags[NUMSMHD];/* fragments queued by async send */
 static int nextsmhd = 0;				/* index of current isend mhdl */
-#if defined(IMA_CM5) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_CM5) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) \
+		defined(IMA_AIX5SP2)
 static struct pmsg *precvlist = 0;		/* not-processed incm msgs */
 #endif
 static long pvmmyptype = 0;				/* my process type */
@@ -461,7 +469,7 @@ mroute(mid, dtid, code, tmout)
 	struct timeval tnow, tstop;
 	int len;
 	long node;
-#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	MPI_Status info;
 	int mpiflag = 0;
 	int mpisiz;
@@ -654,12 +662,13 @@ node_send(txup, txfp, dtid, code)
 	int i;
 	int ff;
 	char dummy[TDFRAGHDR+TTMSGHDR];	/* for inplace data */
-#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	MPI_Status mpista;
 	int mpiflag = 0;
 #endif
 
-#if defined(IMA_PGON) || defined(IMA_CM5) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_PGON) || defined(IMA_CM5) || defined(IMA_SP2MPI) \
+		|| defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	mask |= pvmtidtmask;				/* process type */
 #endif
 
@@ -767,7 +776,7 @@ node_send(txup, txfp, dtid, code)
 		} else {
 
 			if (node != pvmhostnode) {
-#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 				if (mpierrcode = MPI_Isend(txcp, txtogo, MPI_BYTE, node, 
 				PMTPACK, MPI_COMM_WORLD, &outfrags[nextsmhd].tf_mhdl)) {
 #else
@@ -944,7 +953,8 @@ node_mcast(mid, dtid, code)
 	PVM_FREE(nodes);
 #endif /*defined(IMA_PGON)*/
 
-#if defined(IMA_CM5) || defined(IMA_I860) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_CM5) || defined(IMA_I860) || defined(IMA_SP2MPI) \
+		|| defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	for (i = 0; i < ntask; i++)
 		cc = mroute(mid, tids[i], code, &ztv);
 #endif
@@ -1012,7 +1022,7 @@ msendrecv(other, code)
 }
 
 
-#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 
 /* Relay messages between pvmd and node tasks. */
 void relay(dsock)
@@ -1293,7 +1303,7 @@ pvmbeatask()
 	char *p;
 	struct pvmminfo minfo;
 	int n;
-#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	char *msgbuf;				/* buffer for Bsend */
 	int msgbufsiz;
 #endif
@@ -1343,7 +1353,7 @@ pvmbeatask()
 	pvmhostnode = CMMD_host_node();
 #endif
 
-#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	MPI_Init(&ac, NULL);
 	MPI_Comm_rank(MPI_COMM_WORLD, &pvmmynode);
 	MPI_Comm_size(MPI_COMM_WORLD, &pvmhostnode);
@@ -1390,7 +1400,8 @@ fflush(stdout);
 	BZERO((char*)pvmrxlist, sizeof(struct pmsg));
 	pvmrxlist->m_link = pvmrxlist->m_rlink = pvmrxlist;
 
-#if defined(IMA_CM5) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_CM5) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) \
+		|| defined(IMA_AIX5SP2)
 	precvlist = TALLOC(1, struct pmsg, "umb");
 	BZERO((char*)precvlist, sizeof(struct pmsg));
 	precvlist->m_link = precvlist->m_rlink = precvlist;
@@ -1403,7 +1414,8 @@ fflush(stdout);
 	_msgwait(rmid);
 #endif
 
-#if defined(IMA_PGON) || defined(IMA_I860) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_PGON) || defined(IMA_I860) || defined(IMA_SP2MPI) \
+		|| defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	if (pvminfo[0] != TDPROTOCOL) {
 		sprintf(pvmtxt, "beatask() t-d protocol mismatch (%d/%d)\n",
 			TDPROTOCOL, pvminfo[0]);
@@ -1559,9 +1571,10 @@ fflush(stdout);
 int
 pvmendtask()
 {
-#if defined(IMA_PGON) || defined(IMA_I860) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_PGON) || defined(IMA_I860) || defined(IMA_SP2MPI) \
+		|| defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	int i;
-#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	MPI_Status mpista;
 #endif
 
@@ -1577,7 +1590,7 @@ pvmendtask()
 #ifdef IMA_CM5
 	CMMD_all_msgs_wait();
 #endif
-#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	MPI_Finalize();
 #endif
 
@@ -1629,7 +1642,7 @@ mpprecv(tid, tag, cp, len, rtid, rtag, rlen)
 #if defined(IMA_PGON) || defined(IMA_CM5)
 	long info[8];
 #endif
-#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 	MPI_Status info;
 	int mpiflag = 0;
 	int mpisiz;
@@ -1639,7 +1652,8 @@ mpprecv(tid, tag, cp, len, rtid, rtag, rlen)
 	int cc;
 
 	node = (tid == -1) ? MPPANYNODE : tid & pvmtidnmask;
-#if defined(IMA_CM5) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_CM5) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) \
+		|| defined(IMA_AIX5SP2)
 	for (up = precvlist->m_link; up != precvlist; up = up->m_link)
 		/* message picked up by psend */
 		if ((tag == -1 || tag == up->m_tag) 
@@ -1803,7 +1817,8 @@ pvm_precv(tid, tag, cp, len, dt, rtid, rtag, rlen)
 	}
 
 	if (!cc) {
-#if defined(IMA_PGON) || defined(IMA_CM5) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_PGON) || defined(IMA_CM5) || defined(IMA_SP2MPI) \
+		|| defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 		int mask = pvmtidhmask | pvmtidtmask;		/* same partition */
 
 		if (tid == -1 || (TIDISNODE(tid) && (tid & mask) == (pvmmytid & mask)))
@@ -1939,7 +1954,8 @@ pvm_psend(tid, tag, cp, len, dt)
 
 	if (!cc) {
 
-#if defined(IMA_PGON) || defined(IMA_CM5) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_PGON) || defined(IMA_CM5) || defined(IMA_SP2MPI) \
+		|| defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 
 		int mask = pvmtidhmask;     		/* host */
 		long node = tid & pvmtidnmask;
@@ -1947,7 +1963,7 @@ pvm_psend(tid, tag, cp, len, dt)
 		CMMD_mcb mhdl;
 		int info;
 #endif
-#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 		MPI_Request mhdl;
 		MPI_Status info, mpista;
 		int mpiflag = 0;
@@ -1968,7 +1984,7 @@ pvm_psend(tid, tag, cp, len, dt)
 #ifdef IMA_CM5
 			if ((mhdl = ASYNCSEND(tag, cp, len, node, pvmmyptype)) < 0)
 #endif
-#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 			if (mpierrcode = MPI_Isend(cp, len, MPI_BYTE, node,
                 tag, MPI_COMM_WORLD, &mhdl))
 #endif
@@ -1977,7 +1993,8 @@ pvm_psend(tid, tag, cp, len, dt)
 				cc = PvmSysErr;
 				goto done;
 			}
-#if defined(IMA_CM5) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2)
+#if defined(IMA_CM5) || defined(IMA_SP2MPI) || defined(IMA_AIX4SP2) \
+		|| defined(IMA_AIX5SP2)
 			while (!ASYNCDONE(mhdl)) {
 				int rtag;
 
