@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: lpvmpack.c,v 1.17 1999/08/12 15:12:19 pvmsrc Exp $";
+	"$Id: lpvmpack.c,v 1.21 2001/02/07 23:14:08 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -36,6 +36,30 @@ static char rcsid[] =
  *	Typed packing/unpacking, message buffer manip.
  *
  * $Log: lpvmpack.c,v $
+ * Revision 1.21  2001/02/07 23:14:08  pvmsrc
+ * First Half of CYGWIN Check-ins...
+ * (Spanker=kohl)
+ *
+ * Revision 1.20  2001/02/05 23:05:55  pvmsrc
+ * Fixed va_arg() issue.
+ * 	- shouldn't use "char" or "short", always use at least "int"...
+ * 	- error reported by gcc on latest Linux, documentation also
+ * 		indicates this guideline on SGI and other arches...
+ * 	- new test suite test validates functionality on SGI (of course,
+ * 		it worked before, too :-).
+ * 	- here's hoping this won't break anything...  :-o
+ * (Spanker=kohl)
+ *
+ * Revision 1.19  2000/02/16 21:59:46  pvmsrc
+ * Fixed up #include <sys/types.h> stuff...
+ * 	- use <bsd/sys/types.h> for IMA_TITN...
+ * 	- #include before any NEEDMENDIAN #includes...
+ * (Spanker=kohl)
+ *
+ * Revision 1.18  1999/11/08 17:44:34  pvmsrc
+ * SGI compiler cleanup.
+ * (Spanker=kohl)
+ *
  * Revision 1.17  1999/08/12 15:12:19  pvmsrc
  * Fixed error in arg to check_for_exit().
  * 	- mp freed before mp->m_src passed in...!  D-Oh!
@@ -171,6 +195,7 @@ static char rcsid[] =
 #endif
 #include <stdio.h>
 #ifdef NEEDMENDIAN
+#include <sys/types.h>
 #include <machine/endian.h>
 #endif
 #ifdef NEEDENDIAN
@@ -181,12 +206,10 @@ static char rcsid[] =
 #endif
 
 #include <pvm3.h>
-#ifdef WIN32
-#ifndef _XDR_HEADERS
-#define _XDR_HEADERS
+
+#if defined(WIN32) || defined(CYGWIN)
 #include "..\xdr\types.h"
 #include "..\xdr\xdr.h"
-#endif
 #else
 #include <rpc/types.h>
 #include <rpc/xdr.h>
@@ -407,12 +430,11 @@ umbuf_list(lvl)
 	int lvl;
 {
 	int i;
-	struct pmsg *mp;
 	struct frag *fp;
 	int rlen;
 
 	for (i = 1; i < pvmmidhsiz; i++)
-		if (mp = pvmmidh[i].m_umb)
+		if (pvmmidh[i].m_umb)
 			umbuf_dump(i, lvl);
 	return 0;
 }
@@ -1814,7 +1836,7 @@ pvm_vpackf(fmt, ap)
 
 			case 'c':
 				if (isv) {
-					tc = va_arg(ap, char);
+					tc = (char) va_arg(ap, int);
 					cp = &tc;
 				} else
 					cp = va_arg(ap, char *);
@@ -1822,6 +1844,8 @@ pvm_vpackf(fmt, ap)
 #ifdef	DEBUGPACKF
 				printf("%d %d %s%schar\n", cnt, std, (vu ? "unsigned " : ""),
 						(isv ? "" : "&"));
+#else
+				vu = vu; /* sgi compiler */
 #endif
 				break;
 
@@ -1837,7 +1861,7 @@ pvm_vpackf(fmt, ap)
 				} else
 					if (vh) {
 						if (isv) {
-							th = va_arg(ap, short);
+							th = (short) va_arg(ap, int);
 							hp = &th;
 						} else
 							hp = va_arg(ap, short *);
