@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: lpvmshmem.c,v 1.42 1999/07/08 19:00:02 kohl Exp $";
+	"$Id: lpvmshmem.c,v 1.47 2004/01/14 18:50:57 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -36,6 +36,30 @@ static char rcsid[] =
  *	Libpvm core for MPP environment.
  *
  * $Log: lpvmshmem.c,v $
+ * Revision 1.47  2004/01/14 18:50:57  pvmsrc
+ * Added new AIX5* arches.
+ * (Spanker=kohl)
+ *
+ * Revision 1.46  2002/02/18 19:18:35  pvmsrc
+ * Added "|| defined(IMA_SUN4SOL2)" to "const void *i, *j" #if
+ * 	for int_compare() / qsort()...
+ * 	(needed for sure for 64 bit) Solaris)
+ * (Spanker=kohl)
+ *
+ * Revision 1.45  2001/09/25 21:21:00  pvmsrc
+ * Yanked "char *pvmgettmp();" decl - now in pvm3.h...
+ * (Spanker=kohl)
+ *
+ * Revision 1.44  2001/05/11 17:32:29  pvmsrc
+ * Eliminated references to sys_errlist & sys_nerr.
+ * 	- unnecessary, and we're whacking that crap anyway.
+ * (Spanker=kohl)
+ *
+ * Revision 1.43  2000/02/10 20:44:41  pvmsrc
+ * Replaced hard-coded /tmp for PVMSHMFILE.
+ * 	- use pvmgettmp() routine now to determine PVM temp dir.
+ * (Spanker=kohl)
+ *
  * Revision 1.42  1999/07/08 19:00:02  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -478,6 +502,7 @@ static char rcsid[] =
 #endif
 
 char *getenv();
+
 void hex_inadport __ProtoGlarp__ (( char *, struct sockaddr_in * ));
 
 /* These prototypes stop compiler casting structs to ints... */
@@ -503,8 +528,6 @@ int current_node();
 
 #ifndef HASERRORVARS
 extern int errno;					/* from libc */
-extern char *sys_errlist[];
-extern int sys_nerr;
 #endif
 
 /* XXX this should be in a header file not copied */
@@ -667,7 +690,7 @@ peer_wait()
 #if defined(IMA_SUNMP) && defined(PVMUSEMUTEX)
 		cond_wait(&inbp->mb_cond, &inbp->mb_lock);
 #endif
-#if defined(IMA_RS6KMP) || defined(IMA_AIX4MP)
+#if defined(IMA_RS6KMP) || defined(IMA_AIX4MP) || defined(IMA_AIX5MP)
 		pthread_cond_wait(&inbp->mb_cond, &inbp->mb_lock);
 #endif
 	PAGEUNLOCK(&inbp->mb_lock);
@@ -1249,12 +1272,12 @@ peer_send(txup, txfp, dtid, code)
 
 	if (dboxp != (struct msgboxhdr *)pvmdinbox && dboxp->mb_sleep) {
 
-/* #if	defined(IMA_SUNMP) || defined(IMA_RS6KMP) || defined(IMA_AIX4MP) */
+/* #if	defined(IMA_SUNMP) || defined(IMA_RS6KMP) || defined(IMA_AIX4MP) || defined(IMA_AIX5MP) */
 #ifdef  PVMUSEMUTEX
 #ifdef	IMA_SUNMP
 		cond_signal(&dboxp->mb_cond);
 #endif
-#if defined(IMA_RS6KMP) || defined(IMA_AIX4MP)
+#if defined(IMA_RS6KMP) || defined(IMA_AIX4MP) || defined(IMA_AIX5MP)
 		pthread_cond_signal(&dboxp->mb_cond);
 #endif
 #else
@@ -1463,6 +1486,7 @@ pvmbeatask()
 #endif
 	int key, firstkeytried;
 	int mytid;
+	char *pvmtmp;
 	TEV_DECLS
 
 	if (pvmmytid != -1)
@@ -1491,11 +1515,13 @@ pvmbeatask()
 #ifdef IMA_CSPP
 	int scid = get_scid();
 
+	pvmtmp = pvmgettmp();
+
 	if (scid > 1)
-		sprintf(fname, "/tmp/pvmt.%d.%d", pvm_useruid, scid);
+		sprintf(fname, "%s/pvmt.%d.%d", pvmtmp, pvm_useruid, scid);
 	else
 #endif
-		sprintf(fname, "/tmp/pvmt.%d", pvm_useruid);
+		sprintf(fname, "%s/pvmt.%d", pvmtmp, pvm_useruid);
 		if ((logfp = fopen(fname, "a")) == NULL)
 			pvmlogerror("pvmbeatask() can't open log file\n");
 #endif
@@ -2313,6 +2339,7 @@ dynbuf(tid, len)
 	int fd;
 	char fname[32];
 	struct shmpghdr *ph;
+	char *pvmtmp;
 
 	while (!(pp = peer_conn(tid, (int *)0)))
 		;
@@ -2323,7 +2350,8 @@ dynbuf(tid, len)
 	}
 
 	if (!(ph = (struct shmpghdr *)pp->p_dbuf)) {
-		sprintf(fname, PVMSHMFILE, tid);
+		pvmtmp = pvmgettmp();
+		sprintf(fname, PVMSHMFILE, pvmtmp, tid);
 		if ((fd = open(fname, O_CREAT|O_RDWR, 0600)) == -1 ||
 		(pp->p_dbuf = (char *)mmap(0, max(len,SHMBUFSIZE), PROT_READ|PROT_WRITE,
 #if defined(IMA_SGIMP) || defined(IMA_SGIMP6) || defined(IMA_SGIMP64)
@@ -2478,7 +2506,7 @@ pvm_psend(tid, tag, cp, len, dt)
 
 static int
 int_compare(i, j)
-#if defined(IMA_SGI) || defined(IMA_SGI5) || defined(IMA_SGI6) || defined(IMA_SGI64) || defined(IMA_SGIMP) || defined(IMA_SGIMP6) || defined(IMA_SGIMP64)
+#if defined(IMA_SGI) || defined(IMA_SGI5) || defined(IMA_SGI6) || defined(IMA_SGI64) || defined(IMA_SGIMP) || defined(IMA_SGIMP6) || defined(IMA_SGIMP64) || defined(IMA_SUN4SOL2)
 	const void *i, *j;
 #else
 	void *i, *j;
@@ -2766,7 +2794,9 @@ shmemuptod(txup)
 	/* in trouble anyway! */
 	while (txtogo) {
 
-#if defined(IMA_RS6K) || defined(IMA_SP2MPI) || defined(IMA_AIX4) || defined (IMA_AIX4MP) || defined(IMA_AIX4SP2)
+#if defined(IMA_RS6K) || defined(IMA_SP2MPI) || defined(IMA_AIX4) \
+		|| defined (IMA_AIX4MP) || defined(IMA_AIX5MP) \
+		|| defined(IMA_AIX4SP2) || defined(IMA_AIX5SP2)
 		n = write(pvmdsock, txcp, min(txtogo, 4096));
 #else
 		n = write(pvmdsock, txcp, txtogo);
@@ -2828,7 +2858,8 @@ int
 sleep_dammit( msecs )
 int msecs;
 {
-#if defined(IMA_SUNMP) || defined(IMA_RS6KMP) || defined(IMA_AIX4MP)
+#if defined(IMA_SUNMP) || defined(IMA_RS6KMP) || defined(IMA_AIX4MP) \
+		defined(IMA_AIX5MP)
 	return( usleep( msecs ) );
 #else
 	struct timeval biteme;
