@@ -1,12 +1,16 @@
 
+static char rcsid[] =
+	"$Id: pvmalloc.c,v 1.4 1999/07/08 19:00:06 kohl Exp $";
+
 /*
- *         PVM version 3.3:  Parallel Virtual Machine System
+ *         PVM version 3.4:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
- *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
- *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
- *                   (C) 1992 All Rights Reserved
+ *      Authors:  J. J. Dongarra, G. E. Fagg, M. Fischer
+ *          G. A. Geist, J. A. Kohl, R. J. Manchek, P. Mucci,
+ *         P. M. Papadopoulos, S. L. Scott, and V. S. Sunderam
+ *                   (C) 1997 All Rights Reserved
  *
  *                              NOTICE
  *
@@ -31,7 +35,21 @@
  *
  *	Malloc filter interface.
  *
-$Log: pvmalloc.c,v $
+ * $Log: pvmalloc.c,v $
+ * Revision 1.4  1999/07/08 19:00:06  kohl
+ * Fixed "Log" keyword placement.
+ * 	- indent with " * " for new CVS.
+ *
+ * Revision 1.3  1997/06/25  22:09:18  pvmsrc
+ * Markus adds his frigging name to the author list of
+ * 	every file he ever looked at...
+ *
+ * Revision 1.2  1997/01/28  19:26:59  pvmsrc
+ * New Copyright Notice & Authors.
+ *
+ * Revision 1.1  1996/09/23  23:44:28  pvmsrc
+ * Initial revision
+ *
  * Revision 1.2  1994/06/03  20:38:21  manchek
  * version 3.3.0
  *
@@ -40,10 +58,7 @@ $Log: pvmalloc.c,v $
  *
  */
 
-/*
-#include <malloc.h>
-*/
-#include "protoglarp.h"
+#include <pvm3.h>
 #include "pvmalloc.h"
 
 char *i_malloc(), *i_realloc();
@@ -53,9 +68,6 @@ char *i_malloc(), *i_realloc();
  **           **
  ***************/
 
-static char rcsid[] = "$Id: pvmalloc.c,v 1.2 1994/06/03 20:38:21 manchek Exp $";
-static char pvmtxt[512];		/* scratch for error log */
-
 
 /*****************
  **  Interface  **
@@ -70,8 +82,7 @@ pvm_alloc(n, tag)
 	char *r;
 
 	if (!(r = i_malloc(n, tag))) {
-		sprintf(pvmtxt, "pvm_alloc: can't get %d bytes", n);
-		pvmlogerror(pvmtxt);
+		pvmlogprintf("pvm_alloc: can't get %d bytes", n);
 	}
 	return r;
 }
@@ -85,8 +96,7 @@ pvm_realloc(p, n)
 	char *r;
 
 	if (!(r = i_realloc(p, n))) {
-		sprintf(pvmtxt, "pvm_realloc: can't get %d bytes", n);
-		pvmlogerror(pvmtxt);
+		pvmlogprintf("pvm_realloc: can't get %d bytes", n);
 	}
 	return r;
 }
