@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: pvmcruft.c,v 1.25 1999/07/08 19:00:06 kohl Exp $";
+	"$Id: pvmcruft.c,v 1.40 2004/03/10 17:01:27 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -36,6 +36,89 @@ static char rcsid[] =
  *	Missing links and other wonk.
  *
  * $Log: pvmcruft.c,v $
+ * Revision 1.40  2004/03/10 17:01:27  pvmsrc
+ * Fixed funny pvmtmpnam() bug...  (ha ha ha :)
+ * 	- first pvmd call for socket file will name clash after 10000
+ * 		task spawns (t-auth/d-auth)...  (go figure... :-)
+ * 	- tweaked to start counter at 0, but thereafter roll over back to 1.
+ * (Spanker=kohl)
+ *
+ * Revision 1.39  2002/04/23 16:33:29  pvmsrc
+ * Added overflow check in pvmtmpnam() for PVMTMPNAMLEN string length...
+ * 	- just a good idea to check, though it's a lazy check-and-bail...
+ * (Spanker=kohl)
+ *
+ * Revision 1.38  2001/09/27 21:25:10  pvmsrc
+ * BEOSCYLD port.
+ * 	- submitted by Joe Vitale <vitale@scyld.com>.
+ * 	(we renamed it from BEOWULF to BEOSCYLD, but it's his port... :-)
+ * (Spanker=kohl)
+ *
+ * Revision 1.37  2001/09/26 22:04:20  pvmsrc
+ * Modified temp file prefix in pvmtmpnam().
+ * 	- tmp* -> pvmtmp*.
+ * (Spanker=kohl)
+ *
+ * Revision 1.36  2001/09/26 21:22:59  pvmsrc
+ * Append VMID string to PVMD socket file name in pvmdsockfile().
+ * 	- if set in PVM_VMID env var.
+ * 	- not restricted to be integer as in SGI, can be arbitrary
+ * 		string...  heh, heh...  :-}
+ * (Spanker=kohl)
+ *
+ * Revision 1.35  2001/09/25 21:21:02  pvmsrc
+ * Yanked "char *pvmgettmp();" decl - now in pvm3.h...
+ * (Spanker=kohl)
+ *
+ * Revision 1.34  2001/02/09 15:23:47  pvmsrc
+ * Minor tweak to pvmgettmp() for Markus.
+ * 	- check %PVM_TMP% & registry before resorting to %TEMP%.
+ * (Spanker=kohl)
+ *
+ * Revision 1.33  2001/02/07 23:15:51  pvmsrc
+ * 2nd Half of CYGWIN Check-ins...
+ * (Spanker=kohl)
+ *
+ * Revision 1.32  2001/02/02 14:50:26  pvmsrc
+ * Win32 fixes & additions.
+ * (Spanker=kohl)
+ *
+ * Revision 1.31  2000/03/29 20:04:12  pvmsrc
+ * Added prototype for pvm_readregistry() routine.
+ * (Spanker=sscott)
+ *
+ * Revision 1.30  2000/02/17 23:12:15  pvmsrc
+ * *** Changes for new BEOLIN port ***
+ * 	- MPP-like, similar to SP2, etc.
+ * 	- submitted by Paul Springer <pls@smokeymt.jpl.nasa.gov>.
+ * 	- format-checked & cleaned up by Jeembo...  :-)
+ * (Spanker=kohl)
+ *
+ * Revision 1.29  2000/02/16 23:30:24  pvmsrc
+ * Added back in read_pvmregistry() calls.
+ * 	- for PVM_ROOT and PVM_TMP, if env vars not present on WIN32.
+ * 	- couldn't find reference to PVM_ARCH anywhere, Markus?!
+ * (Spanker=kohl)
+ *
+ * Revision 1.28  2000/02/14 20:32:11  pvmsrc
+ * Added new pvmgetrsh() routine
+ * 	- checks for PVM_RSH or else uses old RSHCOMMAND interface.
+ * 	- one-stop shopping for rsh...
+ * (Spanker=kohl)
+ *
+ * Revision 1.27  2000/02/10 20:45:15  pvmsrc
+ * Replaced hard-coded /tmp usage.
+ * 	- modified pvmgettmp() to try $PVM_TMP env var for Unix, too.
+ * 	- use for TDSOCKNAME / TDSOCKNAME_CSPP in pvmdsockfile().
+ * Also in pvmdsockfile():
+ * 	- fixed WIN32 chdir() problem, cleaned up string manip.
+ * (Spanker=kohl)
+ *
+ * Revision 1.26  1999/12/13 18:13:55  pvmsrc
+ * Moved pvmmatchstring() here from msgbox.c.
+ * 	- for use by pvmtester stuff...
+ * (Spanker=kohl)
+ *
  * Revision 1.25  1999/07/08 19:00:06  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -212,19 +295,30 @@ static char rcsid[] =
 #include "lpvm.h"
 #include "global.h"
 
+#ifdef WIN32
+char *read_pvmregistry	__ProtoGlarp__(( const char * ));
+#endif
 
 /***************
  **  Globals  **
  **           **
  ***************/
 
-char *pvmgettmp();
+#ifndef RSHCOMMAND
+#define RSHCOMMAND  "/usr/ucb/rsh"
+#endif
+
 char *getenv();
 
 #ifdef WIN32
 extern char *username;
 #endif
 
+#ifdef IMA_BEOSCYLD
+#include <sys/bproc.h>
+#endif
+
+
 /***************
  **  Private  **
  **           **
@@ -640,6 +734,7 @@ char *
 pvmtmpnam(buf)
 	char *buf;
 {
+	static int f = 0;
 	static int n = 0;
 	static char scratch[255];
 
@@ -650,13 +745,54 @@ pvmtmpnam(buf)
 
 	pvmtmp = pvmgettmp();
 
-	sprintf(buf, "%s/tmp%06d.%d", pvmtmp, getpid(), n++ % 10000);
+#ifndef WIN32
+	sprintf(buf, "%s/pvmtmp%06d.%d", pvmtmp, getpid(), n);
+#else
+	sprintf(buf, "%s\\pvmtmp%06d.%d", pvmtmp, getpid(), n);
+#endif
+
+	/* Silly Hack to avoid stomping on pvmd's first pvmtmpnam()
+		call to get it's socket file... :-) */
+	n = 1 + (f++ % 10000);
+
+	/* Better Check for String Overflow... */
+	/* (Late Check is Better Than No Check... :-) */
+
+	if ( strlen( buf ) >= PVMTMPNAMLEN ) {
+		pvmlogperror("Whoa!  PVMTMPNAMLEN Overflow in pvmtmpnam()...");
+		pvmlogprintf("<%s> = %d >= %d", buf, strlen( buf ),
+				PVMTMPNAMLEN);
+		pvmbailout(0);
+		exit(1);		/* the other meaning of bail out */
+	}
 
 	return buf;
 }
 #endif	/*NOTMPNAM*/
 
 
+#ifdef IMA_BEOSCYLD
+
+/*
+ * Implement a version of gethostname() that gives the "right" answer
+ * for a Scyld Beowulf cluster; calling bproc_currnode() here is why
+ * both the pvm daemon and the pvm library are linked with libbproc
+ */
+
+int
+gethostname(name, len)
+	char * name;
+	size_t len;
+{
+	int retval;
+
+	retval = snprintf(name,len,".%d",bproc_currnode());
+
+	return( (retval == -1) ? retval : 0 );
+}
+
+#endif
+
 pvmhdump(cp, n, pad)
 	char *cp;		/* bytes */
 	int n;			/* length */
@@ -752,10 +888,18 @@ pvmgetroot()
 
 #else
 
+#ifdef CYGWIN
+		rd = getenv("PVM_ROOT_U");
+#else
 		rd = getenv("PVM_ROOT");
+#endif
+
+#ifdef WIN32
+		if (!rd)
+			rd = read_pvmregistry("PVM_ROOT");
+#endif
 
 #endif
-/* IMA_CSPP */
 
 		if (!rd) {
 			pvmlogerror("PVM_ROOT environment variable not set.\n");
@@ -780,14 +924,24 @@ pvmgettmp()
 
 	if (!td) {
 #ifndef WIN32
-		td = "/tmp";
+		if ( !(td=getenv("PVM_TMP")) )
+#ifdef IMA_BEOLIN
+			td = "/tmps";	/* the shared tmp */
 #else
-		if ( !(td=getenv("PVM_TMP")) && !(td=getenv("TEMP")) ) {
-			td = "C:\\TEMP";
-			fprintf(stderr,
-					"Could not get %%PVM_TMP%% or %%TEMP%% ");
-			fprintf(stderr, "- using \"%s\".\n", td);
-			fprintf(stderr, "Please check your installation.\n");
+			td = "/tmp";
+#endif
+#else
+		if ( !(td=getenv("PVM_TMP")) ) {
+			if ( !(td=read_pvmregistry("PVM_TMP")) ) {
+				if ( !(td=getenv("TEMP")) ) {
+					td = "C:\\TEMP";
+					fprintf(stderr,
+							"Could not get %%PVM_TMP%% or %%TEMP%% ");
+					fprintf(stderr, "-> using \"%s\".\n", td);
+					fprintf(stderr,
+							"Please check your PVM installation.\n");
+				}
+			}
 		}
 #endif
 	}
@@ -796,6 +950,25 @@ pvmgettmp()
 }
 
 
+/*	pvmgetrsh()
+*
+*	Return absolute path of the rsh executable for the local system.
+*/
+
+char *
+pvmgetrsh()
+{
+	static char *rd = 0;
+
+	if (!rd) {
+		if ( !(rd=getenv("PVM_RSH")) )
+			rd = RSHCOMMAND;
+	}
+
+	return rd;
+}
+
+
 /*	pvmgetpvmd()
 *
 *	Return absolute path of pvmd startup script.
@@ -829,16 +1002,18 @@ pvmgetpvmd()
 char *
 pvmdsockfile()
 {
-	static char buf[160];
-#ifdef WIN32
-	char TDSOCKSPEC[256];
-#endif
+	static char buf[255];
+
 	char hna[128];
+	char *pvmtmp;
 	char *p;
+
 #ifdef	IMA_CSPP
 	int scid = get_scid();	/* default (system) subcomplex ID is 1 */
 #endif
 
+	pvmtmp = pvmgettmp();
+
 #ifdef	SHAREDTMP
 
 	if (gethostname(hna, sizeof(hna)-1) == -1) {
@@ -850,42 +1025,33 @@ pvmdsockfile()
 
 #ifdef	IMA_CSPP
 	if (scid > 1)
-		(void)sprintf(buf, TDSOCKNAME_CSPP, pvm_useruid, scid, hna);
+		(void)sprintf(buf, TDSOCKNAME_CSPP,
+				pvmtmp, pvm_useruid, scid, hna);
 	else
 #endif
-		(void)sprintf(buf, TDSOCKNAME, pvm_useruid, hna);
+		(void)sprintf(buf, TDSOCKNAME, pvmtmp, pvm_useruid, hna);
 
 #else
 
 #ifdef	IMA_CSPP
 	if (scid > 1)
-		(void)sprintf(buf, TDSOCKNAME_CSPP, pvm_useruid, scid);
+		(void)sprintf(buf, TDSOCKNAME_CSPP, pvmtmp, pvm_useruid, scid);
 	else
 #endif
 #ifdef WIN32
-
-		p = pvmgettmp();
-
-		if (chdir(p) == -1) {
-			pvmlogprintf(
-				"Could Not Retrieve PVM Temporary Directory \"%s\".\n",
-				p);
-			pvmbailout(0);
-		}
-
-		strcpy(TDSOCKSPEC, p);
-
-		sprintf(buf,"\\pvmd.%s",username);
-		strcat(TDSOCKSPEC,buf);
-
-		sprintf(buf,TDSOCKSPEC);
-
+		sprintf( buf, "%s\\pvmd.%s", pvmtmp, username );
 #else
-		(void)sprintf(buf, TDSOCKNAME, pvm_useruid);
+		(void)sprintf(buf, TDSOCKNAME, pvmtmp, pvm_useruid);
 #endif
 
 #endif
 
+	/* Append a Virtual Machine ID, If Set */
+	if ( p = getenv("PVM_VMID") ) {
+		strcat( buf, "." );
+		strcat( buf, p );
+	}
+
 	return buf;
 }
 
@@ -1205,3 +1371,92 @@ int secs;
 	return( ret );
 }
 
+
+/*	pvmmatchstring()
+*
+*	Replacement for GNU Regex pattern matching.
+*/
+
+int
+pvmmatchstring( str, pattern )
+char *str;
+char *pattern;
+{
+	char *start;
+	char *save;
+	char *x;
+	char *y;
+
+	/* Find Start of Search String (Ignore Preceding '*'s) */
+
+	start = pattern;
+
+	while ( *start == '*' && *start != '\0' )
+		start++;
+
+	/* Null Search - Always Matches */
+
+	if ( *start == '\0' )
+		return( 1 );
+
+	/* Initialize Pointers */
+
+	x = str;
+
+	y = start;
+
+	/* Search Line for Start of Search String */
+
+	while ( *x != '\0' )
+	{
+		/* Starting Match Found...  Check it Out */
+
+		if ( *x == *y )
+		{
+			/* Save Continuation Pointer */
+
+			save = x + 1;
+
+			/* Traverse Line Until No Longer Matches */
+
+			while ( *x != '\0' && *y != '\0' && *x == *y )
+			{
+				x++;
+				y++;
+
+				/* Nested '*' Search Expansion */
+
+				if ( *y == '*' )
+				{
+					if ( pvmmatchstring( x, y + 1 ) )
+						return( 1 );
+
+					else
+						break;
+				}
+
+				/* Skip Over '\' for Escaped '*'s */
+
+				else if ( *y == '\\' && *(y+1) == '*' )
+					y++;
+			}
+
+			/* It's a Match! */
+
+			if ( *y == '\0' )
+				return( 1 );
+
+			/* Reset and Continue */
+
+			x = save;
+
+			y = start;
+		}
+
+		else
+			x++;
+	}
+
+	return( 0 );
+}
+
