@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: pvmdshmem.c,v 1.17 1999/07/08 19:00:11 kohl Exp $";
+	"$Id: pvmdshmem.c,v 1.21 2004/01/14 18:51:02 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -36,6 +36,29 @@ static char rcsid[] =
  * Shared-memory MPP interface.
  *
  * $Log: pvmdshmem.c,v $
+ * Revision 1.21  2004/01/14 18:51:02  pvmsrc
+ * Added new AIX5* arches.
+ * (Spanker=kohl)
+ *
+ * Revision 1.20  2002/02/21 23:18:41  pvmsrc
+ * Added new (not to be documented!) PVM_MAX_TASKS env var support.
+ * 	- for Mahir Lokvancic <mahir@math.ufl.edu>.
+ * 	- forcefully limits the number of tasks that can attach to a
+ * 		pvmd, required on Solaris in rare circumstances when hard
+ * 		FD_SETSIZE limit is reached, and all hell breaks loose...
+ * 	- check return for task_new() call, can now produce NULL ptr,
+ * 		indicating PvmOutOfRes...
+ * (Spanker=kohl)
+ *
+ * Revision 1.19  2001/09/25 21:21:04  pvmsrc
+ * Yanked "char *pvmgettmp();" decl - now in pvm3.h...
+ * (Spanker=kohl)
+ *
+ * Revision 1.18  2000/02/10 20:45:33  pvmsrc
+ * Replaced hard-coded /tmp usage.
+ * 	- use pvmgettmp() routine now to determine PVM temp dir.
+ * (Spanker=kohl)
+ *
  * Revision 1.17  1999/07/08 19:00:11  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -331,6 +354,7 @@ void
 ppi_init()
 {
 	struct pidtidhdr *pvminfo;
+	char *pvmtmp = pvmgettmp();
 	char *p;
 	int key;
 
@@ -340,10 +364,10 @@ ppi_init()
 #ifdef IMA_CSPP
 	int scid = get_scid();
 	if (scid > 1)
-		sprintf(fname, "/tmp/pvmt.%d.%d", pvm_useruid, scid);
+		sprintf(fname, "%s/pvmt.%d.%d", pvmtmp, pvm_useruid, scid);
 	else
 #else
-	sprintf(fname, "/tmp/pvmt.%d", pvm_useruid);
+	sprintf(fname, "%s/pvmt.%d", pvmtmp, pvm_useruid);
 #endif
 	logfp = fopen(fname, "w");
 	fclose(logfp);
@@ -546,7 +570,10 @@ mpp_input()
 					pvmlogerror("mpp_input() out of tids?\n");
 					continue;
 				}
-				tp = task_new(tid);
+				if ((tp = task_new(tid)) == NULL) {
+					pvmlogerror("mpp_input() too many tasks?\n");
+					continue;
+				}
 				task_setpid(tp, src);
 				mpp_conn(tp);
 
@@ -698,12 +725,12 @@ mpp_output(tp, pp)
 
 			if (dboxp->mb_sleep) {
 
-/* #if defined(IMA_SUNMP) || defined(IMA_RS6KMP) || defined(IMA_AIX4MP) */
+/* #if defined(IMA_SUNMP) || defined(IMA_RS6KMP) || defined(IMA_AIX4MP) || defined(IMA_AIX5MP) */
 #ifdef PVMUSEMUTEX
 #ifdef	IMA_SUNMP
 				cond_signal(&dboxp->mb_cond);
 #endif
-#if defined(IMA_RS6KMP) || defined(IMA_AIX4MP)
+#if defined(IMA_RS6KMP) || defined(IMA_AIX4MP) || defined(IMA_AIX5MP)
 				pthread_cond_signal(&dboxp->mb_cond);
 #endif
 #else
