@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: pvmlog.c,v 1.15 1999/07/08 19:00:13 kohl Exp $";
+	"$Id: pvmlog.c,v 1.25 2004/05/26 15:56:36 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -36,6 +36,61 @@ static char rcsid[] =
  *	Error logging fac.
  *
  * $Log: pvmlog.c,v $
+ * Revision 1.25  2004/05/26 15:56:36  pvmsrc
+ * Attempt to fix yet-another-x86_64 bug...
+ * 	- looks like varargs stuff is one-time-use-only...
+ * 		-> replace repeated vfprintf() calls with one vsprintf() call...
+ * (Spanker=kohl)
+ *
+ * Revision 1.24  2004/02/09 17:17:02  pvmsrc
+ * Added pvmlogperror debug output for AMD64...
+ * 	- the arch that wouldn't die...  er, run without seg faulting...  :)
+ * (Spanker=kohl)
+ *
+ * Revision 1.23  2004/02/03 21:40:57  pvmsrc
+ * Added check for NULL strerror() return...
+ * 	- to chase down AMD64 problem...
+ * (Spanker=kohl)
+ *
+ * Revision 1.22  2004/01/14 19:12:35  pvmsrc
+ * Added (char *) cast to strerror() assignment...
+ * (Spanker=kohl)
+ *
+ * Revision 1.21  2001/09/26 21:23:56  pvmsrc
+ * Added Handling for Optional Virtual Machine ID.
+ * 	- append virtual machine ID string to pvmd log file, too,
+ * 		if set in PVM_VMID env var.
+ * 	- not restricted to be integer as in SGI, can be arbitrary
+ * 		string...  heh, heh...  :-}
+ * (Spanker=kohl)
+ *
+ * Revision 1.20  2001/09/25 21:21:05  pvmsrc
+ * Yanked "char *pvmgettmp();" decl - now in pvm3.h...
+ * (Spanker=kohl)
+ *
+ * Revision 1.19  2001/05/11 18:58:13  pvmsrc
+ * Added use of new "USESTRERROR" define.
+ * 	- uses strerror() function instead of sys_errlist/sys_nerr
+ * 		(which aren't public globals on some new operating systems).
+ * (Spanker=kohl)
+ *
+ * Revision 1.18  2000/03/30 18:01:18  pvmsrc
+ * Damn.  Max PVMD Log Size Checking Whacked!
+ * 	- everything goes through pvmlogprintf() now, transferred old
+ * 		code from pvmlogerror() to truncate logging...
+ * (Spanker=kohl)
+ *
+ * Revision 1.17  2000/02/10 20:46:43  pvmsrc
+ * Replaced hard-coded /tmp usage with PVMDLOGFILE.
+ * 	- use pvmgettmp() routine now to determine PVM temp dir.
+ * 	- also, eliminated goofy pvmtmpspec stuff for WIN32, cleaned up
+ * 		string manip...
+ * (Spanker=kohl)
+ *
+ * Revision 1.16  1999/12/10 21:29:40  pvmsrc
+ * Added pvmdlogmax = pvmdlogmax for picky SGI compiler warning...  :-Q
+ * (Spanker=kohl)
+ *
  * Revision 1.15  1999/07/08 19:00:13  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -120,11 +175,8 @@ static char rcsid[] =
  *
  */
 
-#ifdef WIN32
-#include "pvmwin.h"
-#endif
-
 #include <stdio.h>
+#include <pvm3.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <time.h>
@@ -149,20 +201,20 @@ static char rcsid[] =
 /* pvmd log file name */
 
 #ifdef	SHAREDTMP
-#define	PVMDLOGFILE	"/tmp/pvml.%d.%s"
+#define	PVMDLOGFILE	"%s/pvml.%d.%s"
 #else
 #ifndef WIN32
-#define	PVMDLOGFILE	"/tmp/pvml.%d"
+#define	PVMDLOGFILE	"%s/pvml.%d"
 #else
-#define PVMDLOGFILE "$TEMP\\pvml.%s"  /* for default */
+#define PVMDLOGFILE "%s\\pvml.%s"  /* for default */
 #endif
 #endif
 
 #ifdef	IMA_CSPP
 #ifdef	SHAREDTMP
-#define	PVMDLOGFILE_CSPP	"/tmp/pvml.%d.%d.%s"
+#define	PVMDLOGFILE_CSPP	"%s/pvml.%d.%d.%s"
 #else
-#define	PVMDLOGFILE_CSPP	"/tmp/pvml.%d.%d"
+#define	PVMDLOGFILE_CSPP	"%s/pvml.%d.%d"
 #endif
 
 int get_scid();
@@ -174,12 +226,11 @@ int get_scid();
  **           **
  ***************/
 
-char *pvmgettmp();
 char *getenv();
 
 #ifndef HASERRORVARS
 extern int errno;					/* from libc */
-#ifndef WIN32
+#ifndef USESTRERROR
 extern int sys_nerr;
 extern char *sys_errlist[];
 #endif
@@ -194,7 +245,6 @@ extern int pvmmyupid;				/* from pvmd.c */
 extern int pvm_useruid;				/* from pvmd.c */
 #ifdef WIN32
 extern char *username;				/* from pvmd.c */
-extern char *pvmtmpspec;			/* from pvmd.c */
 #endif
 
 int log_fd = -1;					/* error log file */
@@ -205,10 +255,12 @@ int log_fd = -1;					/* error log file */
  **           **
  ***************/
 
-static int log_how = 1;				/* how much error logging we can do */
+static int log_how = 1;		/* how much error logging we can do */
 FILE *log_ff = 0;
+static int log_alrdy = 0;	/* how much already written to file */
 static int pvmdlogmax = PVMDLOGMAX;
-static int atnewline = 1;			/* on new log line (XXX ick) */
+static char *toomuch = "\n*** logging truncated\n";
+static int atnewline = 1;	/* on new log line (XXX ick) */
 
 
 /*	pvmsetlog()
@@ -222,15 +274,18 @@ int
 pvmsetlog(how)
 	int how;
 {
-	char buf[160];
+	char buf[255];
 	char hna[128];
+	char *pvmtmp;
 	char *p;
+
 #ifdef	IMA_CSPP
 	int scid = get_scid();	/* default (system) subcomplex ID is 1) */
-							/* differentiate logfile when not on system s/c */
+					/* differentiate logfile when not on system s/c */
 #endif
 
 	if (2 & how & ~log_how) {
+		pvmtmp = pvmgettmp();
 #ifdef	SHAREDTMP
 		if (gethostname(hna, sizeof(hna)-1) == -1) {
 			pvmlogerror("pvmsetlog() can't gethostname()\n");
@@ -240,30 +295,32 @@ pvmsetlog(how)
 			*p = 0;
 #ifdef	IMA_CSPP
 		if (scid > 1)
-			(void)sprintf(buf, PVMDLOGFILE_CSPP, pvm_useruid, scid, hna);
+			(void)sprintf(buf, PVMDLOGFILE_CSPP,
+					pvmtmp, pvm_useruid, scid, hna);
 		else
 #endif
-			(void)sprintf(buf, PVMDLOGFILE, pvm_useruid, hna);
+			(void)sprintf(buf, PVMDLOGFILE, pvmtmp, pvm_useruid, hna);
 
 #else	/*SHAREDTMP*/
 #ifdef	IMA_CSPP
 		if (scid > 1)
-			(void)sprintf(buf, PVMDLOGFILE_CSPP, pvm_useruid, scid);
+			(void)sprintf(buf, PVMDLOGFILE_CSPP,
+					pvmtmp, pvm_useruid, scid);
 		else
 #endif
 #ifndef WIN32
-			(void)sprintf(buf, PVMDLOGFILE, pvm_useruid);
+		(void)sprintf(buf, PVMDLOGFILE, pvmtmp, pvm_useruid);
 #else
-		if(!pvmtmpspec) pvmtmpspec = malloc(128*sizeof(char));
-
-		p = pvmgettmp();
-
-		strcpy(pvmtmpspec, p);
-		strcat(pvmtmpspec,"\\pvml.%s");
-		(void)sprintf(buf,pvmtmpspec,username);	
+		(void)sprintf(buf, "%s\\pvml.%s", pvmtmp, username);	
 #endif		
 #endif	/*SHAREDTMP*/
 
+		/* Append a Virtual Machine ID, If Set */
+		if ( p = getenv("PVM_VMID") ) {
+			strcat( buf, "." );
+			strcat( buf, p );
+		}
+
 		if ((log_fd = open(buf, O_WRONLY|O_TRUNC|O_CREAT|O_APPEND, 0600)) != -1)
 			log_ff = fdopen(log_fd, "a");
 		else
@@ -271,6 +328,8 @@ pvmsetlog(how)
 
 		if (p = getenv("PVMDLOGMAX"))
 			pvmdlogmax = atoi(p);
+
+		pvmdlogmax = pvmdlogmax;  /* for picky SGI compiler warning */
 	}
 
 	if (2 & log_how & ~how) {
@@ -289,9 +348,7 @@ vpvmlogprintf(fmt, ap)
 	char *fmt;
 	va_list ap;
 {
-	static char *toomuch = "\n*** logging truncated\n";
-	static int log_alrdy = 0;	/* how much already written to file */
-
+	char vtmp[255];
 	int cnt = 0;
 	int cc;
 
@@ -303,6 +360,10 @@ vpvmlogprintf(fmt, ap)
 	tmp = localtime(&now);
 #endif	/*TIMESTAMPLOG*/
 
+	/* snag var arg string, in case re-use whacks it... */
+	/* (thank you very much x86_64...  :-Q */
+	vsprintf(vtmp, fmt, ap);
+
 	if (log_how & 1) {
 		if (atnewline) {
 			if (pvmmytid)
@@ -319,34 +380,38 @@ vpvmlogprintf(fmt, ap)
 						tmp->tm_sec);
 #endif	/*TIMESTAMPLOG*/
 		}
-		cc = vfprintf(stderr, fmt, ap);
+		/* cc = vfprintf(stderr, fmt, ap); */
+		cc = fprintf(stderr, vtmp);
 		cnt = ( cc >= 0 ) ? cnt + cc : cc;
 		fflush(stderr);
 	}
 
 	if (log_how & 2) {
-/*
-		XXX we don't update log_alrdy (log_alrdy < PVMDLOGMAX)
-*/
-		if (atnewline) {
-			if (pvmmytid)
-				fprintf(log_ff, "[t%x] ", pvmmytid);
-			else
-				fprintf(log_ff, "[pvmd pid%d] ", pvmmyupid);
+		if (log_alrdy < pvmdlogmax) {
+			if (atnewline) {
+				if (pvmmytid)
+					fprintf(log_ff, "[t%x] ", pvmmytid);
+				else
+					fprintf(log_ff, "[pvmd pid%d] ", pvmmyupid);
 
 #ifdef	TIMESTAMPLOG
-			fprintf(log_ff, "%02d/%02d %02d:%02d:%02d ",
+				fprintf(log_ff, "%02d/%02d %02d:%02d:%02d ",
 						tmp->tm_mon + 1,
 						tmp->tm_mday,
 						tmp->tm_hour,
 						tmp->tm_min,
 						tmp->tm_sec);
 #endif	/*TIMESTAMPLOG*/
-		}
+			}
 
-		cc = vfprintf(log_ff, fmt, ap);
-		cnt = ( cnt >= 0 ) ? ( ( cc >= 0 ) ? cnt + cc : cc ) : cnt;
-		fflush(log_ff);
+			/* cc = vfprintf(log_ff, fmt, ap); */
+			cc = fprintf(log_ff, vtmp);
+			cnt = ( cnt >= 0 ) ? ( ( cc >= 0 ) ? cnt + cc : cc ) : cnt;
+			fflush(log_ff);
+
+			if ((log_alrdy += cnt) >= pvmdlogmax)
+				(void)write(log_fd, toomuch, strlen(toomuch));
+		}
 	}
 
 	atnewline = (fmt[strlen(fmt) - 1] == '\n') ? 1 : 0;
@@ -402,14 +467,30 @@ pvmlogperror(s)
 	char *em;
 #endif
 #endif
+	int i;
 
 #ifndef WIN32
+#ifdef USESTRERROR
+	em = (char *) strerror( errno );
+#else
 	em = ((errno >= 0 && errno < sys_nerr)
 		? sys_errlist[errno] : "Unknown Error");
+#endif
 #else
 	error_nr = GetLastError();
 	sprintf(em,"%d",error_nr);
 #endif
+
+	pvmlogprintf("%s: ", s);
+	pvmlogprintf("errno=%d", errno);
+	pvmlogprintf("\n");
+
+	pvmlogprintf("em=0x%lx\n", (long) em);
+
+	for (i=0; i < 10 ; i++ )
+		pvmlogprintf("[%x/%c]", em[i]);
+	pvmlogprintf("\n");
+
 	pvmlogprintf("%s: %s\n", s, em);
 	return 0;
 }
@@ -431,8 +512,6 @@ pvmlogerror(s)
 	return 0;
 
 #if NOMORELOGERROR
-	static char *toomuch = "\n*** logging truncated\n";
-	static int log_alrdy = 0;		/* how much already written to file */
 	int len = strlen(s);
 	char buf[40];
 	int l;
