@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: pvmshmem.c,v 1.10 1999/07/08 19:00:14 kohl Exp $";
+	"$Id: pvmshmem.c,v 1.11 2004/01/14 18:51:03 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -36,6 +36,10 @@ static char rcsid[] =
  *  Shared-memory stuff.
  *
  * $Log: pvmshmem.c,v $
+ * Revision 1.11  2004/01/14 18:51:03  pvmsrc
+ * Added new AIX5* arches.
+ * (Spanker=kohl)
+ *
  * Revision 1.10  1999/07/08 19:00:14  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -240,7 +244,7 @@ msgbufinit(buf)
 #if defined(IMA_SUNMP) && defined(PVMUSEMUTEX)
 	cond_init(&((struct msgboxhdr *)buf)->mb_cond, USYNC_PROCESS, 0);
 #endif
-#if defined(IMA_RS6KMP) || defined(IMA_AIX4MP)
+#if defined(IMA_RS6KMP) || defined(IMA_AIX4MP) || defined(IMA_AIX5MP)
 	pthread_cond_init(&((struct msgboxhdr *)buf)->mb_cond, NULL);
 #endif
 	PAGEINITLOCK(&((struct msgboxhdr *)buf)->mb_lock);
