@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: tdpro.c,v 1.28 1999/07/08 19:00:16 kohl Exp $";
+	"$Id: tdpro.c,v 1.34 2002/02/21 23:18:42 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -36,6 +36,55 @@ static char rcsid[] =
  *	Entry points for messages from local tasks.
  *
  * $Log: tdpro.c,v $
+ * Revision 1.34  2002/02/21 23:18:42  pvmsrc
+ * Added new (not to be documented!) PVM_MAX_TASKS env var support.
+ * 	- for Mahir Lokvancic <mahir@math.ufl.edu>.
+ * 	- forcefully limits the number of tasks that can attach to a
+ * 		pvmd, required on Solaris in rare circumstances when hard
+ * 		FD_SETSIZE limit is reached, and all hell breaks loose...
+ * 	- check return for task_new() call, can now produce NULL ptr,
+ * 		indicating PvmOutOfRes...
+ * (Spanker=kohl)
+ *
+ * Revision 1.33  2001/09/25 21:20:20  pvmsrc
+ * Minor TMPNAMFUN()/tmpnam() cleanup.
+ * 	- moved macro def to pvm3.h, renamed PVMTNPMAN().
+ * 	- same for LEN_OF_TMP_NAM -> PVMTMPNAMLEN.
+ * 	- mostly a huge waste of time, since *both* tmpnam() & mktemp()
+ * 		produce the same "dangerous" warning message in Linux/gcc...
+ * 	- damn.
+ * (Spanker=kohl)
+ *
+ * Revision 1.32  2001/04/23 14:16:14  pvmsrc
+ * Added new working directory option to pvm_spawn().
+ * 	- use "where" argument to cram in working directory,
+ * 		e.g. where = "msr.epm.ornl.gov:/home/user/project/bozo"
+ * 	- TM_SPAWN strips out working directory & creates PVMSPAWNWD env var
+ * 	- exectasks() (called by DM_EXEC or SM_EXEC) checks for env var
+ * 		and does a chdir() (even uses getcwd() to reset directory :-).
+ * 	- should not introduce any run-time incompatibility with older
+ * 		PVM releases.
+ * 	- PvmTaskHost or PvmTaskArch flags need not be used, i.e.
+ * 		pvm_spawn( "foo", 0, PvmTaskDefault, ":/tmp", 1, &tid ) works.
+ * (Spanker=kohl)
+ *
+ * Revision 1.31  2001/02/07 23:15:58  pvmsrc
+ * 2nd Half of CYGWIN Check-ins...
+ * (Spanker=kohl)
+ *
+ * Revision 1.30  2000/02/17 23:12:20  pvmsrc
+ * *** Changes for new BEOLIN port ***
+ * 	- MPP-like, similar to SP2, etc.
+ * 	- submitted by Paul Springer <pls@smokeymt.jpl.nasa.gov>.
+ * 	- format-checked & cleaned up by Jeembo...  :-)
+ * (Spanker=kohl)
+ *
+ * Revision 1.29  2000/02/16 22:00:30  pvmsrc
+ * Fixed up #include <sys/types.h> stuff...
+ * 	- use <bsd/sys/types.h> for IMA_TITN...
+ * 	- #include before any NEEDMENDIAN #includes...
+ * (Spanker=kohl)
+ *
  * Revision 1.28  1999/07/08 19:00:16  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -220,6 +269,13 @@ static char rcsid[] =
  *
  */
 
+#include <stdio.h>
+
+#ifdef IMA_TITN
+#include <bsd/sys/types.h>
+#else
+#include <sys/types.h>
+#endif
 
 #ifdef NEEDMENDIAN
 #include <machine/endian.h>
@@ -232,21 +288,21 @@ static char rcsid[] =
 #endif
 
 #include <pvm3.h>
-#ifdef WIN32
-#include <sys/types.h>
-#include <time.h>
-#ifndef _XDR_HEADERS
-#define _XDR_HEADERS
+
+#if defined(WIN32) || defined(CYGWIN)
 #include "..\xdr\types.h"
 #include "..\xdr\xdr.h"
-#endif
 #else
-#include <sys/types.h>
+#include <rpc/types.h>
+#include <rpc/xdr.h>
+#endif
+
+#ifdef WIN32
+#include <time.h>
+#else
 #include <sys/time.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#include <rpc/types.h>
-#include <rpc/xdr.h>
 #endif
 
 #include <fcntl.h>
@@ -256,7 +312,6 @@ static char rcsid[] =
 #include <strings.h>
 #endif
 #include <errno.h>
-#include <stdio.h>
 
 #include <pvmproto.h>
 #include "pvmalloc.h"
@@ -268,25 +323,14 @@ static char rcsid[] =
 #include "bfunc.h"
 #include <pvmtev.h>
 #include "global.h"
+#ifdef IMA_BEOLIN
+#include "pvmdmp.h"
+#endif
 
 #ifndef	min
 #define	min(a,b)	((a)<(b)?(a):(b))
 #endif
 
-#ifdef	NOTMPNAM
-#define	TMPNAMFUN(x)	pvmtmpnam(x)
-#define	LEN_OF_TMP_NAM	32
-char *pvmtmpnam();
-
-#else	/*NOTMPNAM*/
-#define	TMPNAMFUN(x)	tmpnam(x)
-#ifdef	L_tmpnam
-#define	LEN_OF_TMP_NAM	L_tmpnam
-#else
-#define	LEN_OF_TMP_NAM	64
-#endif
-#endif	/*NOTMPNAM*/
-
 
 /***************
  **  Globals  **
@@ -473,7 +517,7 @@ tm_connect(tp, mp)
 	struct pmsg *mp;
 {
 	int ver;						/* task's libpvm t-d proto version */
-	char authfn[LEN_OF_TMP_NAM];	/* t-auth file name */
+	char authfn[PVMTMPNAMLEN];	/* t-auth file name */
 	int d;
 	int cc;
 
@@ -501,6 +545,11 @@ tm_connect(tp, mp)
 		goto bail;
 	}
 
+#ifdef NOPROT
+	tp->t_authnam = TALLOC(PVMTMPNAMLEN, char, "auth");
+	(void)PVMTMPNAMFUN(tp->t_authnam);
+#else
+
 	/*
 	*	write in t-auth file, create empty d-auth file that task
 	*	must write and we'll check later
@@ -518,8 +567,8 @@ tm_connect(tp, mp)
 	}
 	(void)close(d);
 
-	tp->t_authnam = TALLOC(LEN_OF_TMP_NAM, char, "auth");
-	(void)TMPNAMFUN(tp->t_authnam);
+	tp->t_authnam = TALLOC(PVMTMPNAMLEN, char, "auth");
+	(void)PVMTMPNAMFUN(tp->t_authnam);
 
 #ifndef IMA_OS2
 	if ((tp->t_authfd = open(tp->t_authnam, O_RDONLY|O_CREAT|O_TRUNC, 0600))
@@ -533,6 +582,8 @@ tm_connect(tp, mp)
 		goto bail;
 	}
 
+#endif
+
 	/*
 	*	task's turn to auth
 	*/
@@ -613,6 +664,8 @@ tm_conn2(tp, mp)
 	*	check that task could write d-auth file
 	*/
 
+#ifndef NOPROT
+
 	if ((cc = read(tp->t_authfd, &c, 1)) == -1) {
 		pvmlogperror("tm_conn2() can't read d-auth file");
 		return 0;
@@ -626,6 +679,9 @@ tm_conn2(tp, mp)
 	(void)close(tp->t_authfd);
 	tp->t_authfd = -1;
 	(void)unlink(tp->t_authnam);
+
+#endif
+
 	PVM_FREE(tp->t_authnam);
 	tp->t_authnam = 0;
 
@@ -640,12 +696,25 @@ tm_conn2(tp, mp)
 		}
 		tp->t_sched = tp2->t_sched;
 
+#ifdef IMA_BEOLIN
+	} else if ( (tp2 = mpp_find(tp))
+			&& !(tp2->t_flag & (TF_AUTH|TF_CONN)) ) {
+		if (pvmdebmask & PDMTASK) {
+			pvmlogprintf(
+					"tm_conn2() reconnect task t%x pid=%d name=%s\n",
+					tp2->t_tid,tp2->t_pid, tp2->t_a_out );
+		}
+#endif
+
 	} else {
 		if ((tid = tid_new()) < 0) {
 			pvmlogerror("tm_conn2() out of tids?\n");
 			goto bail;		/* XXX should disconnect nicely */
 		}
-		tp2 = task_new(tid);
+		if ((tp2 = task_new(tid)) == NULL) {
+			pvmlogerror("tm_conn2() too many tasks?\n");
+			goto bail;		/* XXX should disconnect nicely */
+		}
 
 		if ( !upkstralloc(mp, &taskname) ) {
 			tp2->t_a_out = taskname;
@@ -708,8 +777,20 @@ tm_conn2(tp, mp)
 	tp2->t_sock = tp->t_sock;
 	tp2->t_sad = tp->t_sad;
 	tp2->t_salen = tp->t_salen;
+
+#ifdef NOPROT
+	if (!TIDISNODE(tp2->t_tid)) {
+		if (pvmdebmask & PDMTASK) {
+			pvmlogprintf( "Setting pid of t%x to %d, same as t%x\n",
+					tp2->t_tid, tp->t_pid, tp->t_tid );
+		}
+		task_setpid(tp2, tp->t_pid);
+	}
+#else
 	if (tp2->t_pid != pid)
 		task_setpid(tp2, pid);
+#endif
+
 	tp2->t_rxp = tp->t_rxp;
 	tp2->t_sched = tp->t_sched;
 	tp->t_sock = -1;	/* tp will be freed by loclinput() */
@@ -969,6 +1050,10 @@ tm_spawn(tp, mp)
 	struct hostd *hp;
 	int hh;
 	int i;
+	char *wd = 0;
+	char *wdenv = 0;
+	char *ptr;
+	int sz;
 
 	/*
 	* unpack spawn command from task
@@ -1001,14 +1086,42 @@ tm_spawn(tp, mp)
 	|| upkuint(mp, &wxp->w_trctag))
 		goto bad;
 
+	/*
+	* extract any working directory string from "where"
+	*/
+
+	ptr = where;
+	while ( *ptr != ':' && *ptr != '\0' )
+		ptr++;
+	if ( *ptr == ':' ) {
+		*ptr++ = '\0';    /* close off actual "where" string */
+		wd = ptr;         /* save ptr to working directory */
+	}
+
+	/*
+	* unpack environment from task
+	*/
+
 	if (upkuint(mp, &wxp->w_nenv))
 		goto bad;
-	wxp->w_env = TALLOC((wxp->w_nenv + 1), char*, "env");
-	BZERO((char*)wxp->w_env, (wxp->w_nenv + 1) * sizeof(char*));
+	sz = wxp->w_nenv + 1 + (wd ? 1 : 0);
+	wxp->w_env = TALLOC(sz, char*, "env");
+	BZERO((char*)wxp->w_env, sz * sizeof(char*));
 	for (i = 0; i < wxp->w_nenv; i++)
 		if (upkstralloc(mp, &wxp->w_env[i]))
 			goto bad;
 
+	/*
+	* add extra env string for working directory
+	*/
+
+	if ( wd ) {
+		sz = strlen(wd) + strlen("PVMSPAWNWD=") + 1;
+		wdenv = TALLOC(sz, char, "wdenv");
+		sprintf( wdenv, "PVMSPAWNWD=%s", wd );
+		wxp->w_env[ (wxp->w_nenv)++ ] = STRALLOC( wdenv );
+	}
+
 	/*
 	* make host set containing hosts (matching where option)
 	*/
@@ -1029,6 +1142,13 @@ tm_spawn(tp, mp)
 				&& !strcmp(where, hp->hd_arch))
 					ht_insert(htp, hp);
 
+#ifdef IMA_BEOLIN
+		/* local pvmd for now */
+		} else if (wxp->w_flags & PvmMppFront) {
+			hp = hosts->ht_hosts[hosts->ht_local];
+			ht_insert(htp, hp);
+#endif
+
 		} else {						/* anywhere */
 			ht_merge(htp, hosts);
 		}
@@ -1053,7 +1173,7 @@ tm_spawn(tp, mp)
 		wxp->w_ptid = tp->t_tid;
 	}
 	else
-		wxp->w_ptid = PvmParentNotSet;	/* indicate that task unset parent */
+		wxp->w_ptid = PvmParentNotSet;	/* indicate task unset parent */
 
 	/*
 	* assign each task to a host
