@@ -1,6 +1,6 @@
 
 static char rcsid[] =
-	"$Id: waitc.c,v 1.8 1999/07/08 19:00:18 kohl Exp $";
+	"$Id: waitc.c,v 1.10 2001/02/07 23:16:00 pvmsrc Exp $";
 
 /*
  *         PVM version 3.4:  Parallel Virtual Machine System
@@ -36,6 +36,16 @@ static char rcsid[] =
  *	Wait context descriptors.
  *
  * $Log: waitc.c,v $
+ * Revision 1.10  2001/02/07 23:16:00  pvmsrc
+ * 2nd Half of CYGWIN Check-ins...
+ * (Spanker=kohl)
+ *
+ * Revision 1.9  2000/02/16 22:00:32  pvmsrc
+ * Fixed up #include <sys/types.h> stuff...
+ * 	- use <bsd/sys/types.h> for IMA_TITN...
+ * 	- #include before any NEEDMENDIAN #includes...
+ * (Spanker=kohl)
+ *
  * Revision 1.8  1999/07/08 19:00:18  kohl
  * Fixed "Log" keyword placement.
  * 	- indent with " * " for new CVS.
@@ -92,6 +102,13 @@ static char rcsid[] =
  */
 
 #include <stdio.h>
+
+#ifdef IMA_TITN
+#include <bsd/sys/types.h>
+#else
+#include <sys/types.h>
+#endif
+
 #ifdef NEEDMENDIAN
 #include <machine/endian.h>
 #endif
@@ -103,23 +120,18 @@ static char rcsid[] =
 #endif
 
 #include <pvm3.h>
-#ifdef WIN32
-#ifndef _XDR_HEADERS
-#define _XDR_HEADERS
+
+#if defined(WIN32) || defined(CYGWIN)
 #include "..\xdr\types.h"
 #include "..\xdr\xdr.h"
-#endif
 #else
 #include <rpc/types.h>
 #include <rpc/xdr.h>
-#include <sys/socket.h>
-#include <sys/time.h>
 #endif
 
-#ifdef IMA_TITN
-#include <bsd/sys/types.h>
-#else
-#include <sys/types.h>
+#ifndef WIN32
+#include <sys/socket.h>
+#include <sys/time.h>
 #endif
 
 #ifdef	SYSVSTR
